package roneditor.actions;

/**
 * Edit action for the properties dialog,
 * where the user can change the place or transition properties.
  */
import org.eclipse.gef.EditPart;
import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editparts.AbstractGraphicalEditPart;
import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IWorkbenchPart;

import roneditor.commands.CreateHLTokenCommand;
import roneditor.dialogs.CreateHLPlaceOrTokenDialog;
import roneditor.objectnetview.commands.EditONPlaceCommand;
import roneditor.objectnetview.edit.parts.ONPlaceEditPart;
import roneditor.ronview.edit.parts.HLPlaceEditPart;
import ronmodel.HLPlace;
import ronmodel.HLPlaceType;
import ronmodel.ONPlace;
import ronmodel.RON;
import _static_help_methods.TokenName;


public class AddTokenAction extends SelectionAction {

	static public String ID = "Add Token";
	private static int onCount = 1;
	private static int rCount = 1;
	
	public AddTokenAction(IWorkbenchPart part) {
		super(part);
		setDescription("Add a token to the selected place");
		setText("Add Token");
	}

	/* (non-Javadoc)
	* @see org.eclipse.jface.action.IAction#run()
	*/
	public void run() {
		Object selection = ((StructuredSelection) getSelection()).getFirstElement();
		
		if (selection instanceof ONPlaceEditPart){
			execute(getAddOnTokenCommand());
		}
		if (selection instanceof HLPlaceEditPart){
//			 create dialog
			final Shell shell = getWorkbenchPart().getSite().getShell();
			RON ron=((HLPlace)((EditPart) selection).getModel()).getRon();
			
			String nameText = "newObjectNet"+onCount;
			while(!TokenName.isAllowedName(nameText,ron,true)){
				onCount++;
				nameText = "newObjectNet"+onCount;
			}
			final String nameTextON = nameText;
			
			nameText = "newRule"+rCount;
			while(!TokenName.isAllowedName(nameText,ron,true)){
				rCount++;
				nameText = "newRule"+rCount;
			}
			final String nameTextRule = nameText;
			
			final CreateHLPlaceOrTokenDialog dialog = new CreateHLPlaceOrTokenDialog(
					shell, SWT.NULL, ron) {
				@Override
				protected void init() {
					
					// configure dialog
					getDialogShell().setText("Create token");

					final Object selection = ((StructuredSelection) getSelection())
							.getFirstElement();
					final HLPlace place = (HLPlace) ((EditPart) selection)
							.getModel();				
					if (place.getType().equals(HLPlaceType.NET_LITERAL)) {
						getNetRadioButton().setSelection(true);
						getRuleRadioButton().setSelection(false);
						getNetRadioButton().setEnabled(false);
						getRuleRadioButton().setEnabled(false);
						getRuleRadioButton().setVisible(false);
						getNetRadioButton().setVisible(false);
						getNameText().setText(nameTextON);
						getDialogShell().setText("Add (net-) token");
						onCount++;
					} else {
						getRuleRadioButton().setSelection(true);
						getNetRadioButton().setSelection(false);
						getNetRadioButton().setEnabled(false);
						getRuleRadioButton().setEnabled(false);
						getRuleRadioButton().setVisible(false);
						getNetRadioButton().setVisible(false);
						getNameText().setText(nameTextRule);
						getDialogShell().setText("Add (rule-) token");
						rCount++;
					}
					getNameText().setFocus();
					super.init();
				}
			};

			final CreateHLTokenCommand command = getAddHLTokenCommand();
			// open dialog
			
			do{			
				dialog.open();
				if (dialog.isCancelled()) {
					return;
				}
			} while(!TokenName.isAllowedName(dialog.getNameTextValue(),ron,false)); //should never be true

			// fill command from editpart with dialog data		
			command.setName(dialog.getNameTextValue());
			execute(command);
		}
	}
		
	/*
	 * Get the command class from the place or transition edit part.
	 */
	public Command getAddOnTokenCommand(){
		Object selection = ((StructuredSelection) getSelection()).getFirstElement();
		
		AbstractGraphicalEditPart editpart = null;
			
		if (selection instanceof ONPlaceEditPart){
			editpart = (ONPlaceEditPart)selection;
			EditONPlaceCommand editCommand = new EditONPlaceCommand();
			ONPlace op;
			editCommand.setONPlace(op=(ONPlace)editpart.getModel());			
			editCommand.setNewName(op.getName());
			editCommand.setNewTokens(op.getTokens()+1);
			return editCommand; 
		}
		return null;		
	}
	
	private CreateHLTokenCommand getAddHLTokenCommand() {
		final Object selection = ((StructuredSelection) getSelection())
				.getFirstElement();
		return (CreateHLTokenCommand) ((EditPart) selection)
				.getCommand(new Request(ID));
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.ui.actions.WorkbenchPartAction#calculateEnabled()
	 */
	protected boolean calculateEnabled() {
			return canPerformAction();
	}


	/*
	 * Returns true if the action could be executed, false otherwise.
	 */
	private boolean canPerformAction() {
		if (getSelectedObjects().isEmpty())
			return false;
	
		if (getSelectedObjects().size()==1){
			Object o = getSelectedObjects().get(0);
			if (!(o instanceof EditPart)){
				return false;
			}
			else{
				EditPart part = (EditPart) o;			
				if (part.getModel() instanceof ONPlace)
					return true;
				else if(part instanceof HLPlaceEditPart){
					return part.understandsRequest(new Request(ID));
				}
				else 
					return false;
			}
		}
		return false;
	}


	/* (non-Javadoc)
	 * @see org.eclipse.gef.ui.actions.WorkbenchPartAction#init()
	 */
	protected void init() {
		super.init();
		setId(ID);
	}

}
