package roneditor.commands;

import java.util.Vector;

import org.eclipse.gef.commands.Command;

import roneditor.AbstractMultiViewerPage;
import roneditor.editor.RONEditor;
import roneditor.ronview.commands.DeleteHLArcCommand;
import ronmodel.HLArc;
import ronmodel.HLNode;
import ronmodel.HLPlace;
import ronmodel.RON;
import ronmodel.Token;

/**
 * @author alex und conny
 * 
 * Command for removing a node from its parent RON. This is undoable.
 */
public class DeleteHLNodeCommand extends Command {

	private static final String DeleteCommand_Label = "Delete high-level node";

	final private HLNode node;
	private Vector<DeleteHLArcCommand> deleteHLArcCommands= new Vector<DeleteHLArcCommand>();
	

	private RON parent;

	public DeleteHLNodeCommand(final HLNode node) {
		super(DeleteHLNodeCommand.DeleteCommand_Label);
		this.node = node;
	}
	
	/**
	 * Create delete-commands for all arcs which are connected with the node.
	 * @param onNode
	 */
	private void createDeleteHLArcCommands(HLNode hlNode) {
		deleteHLArcCommands.clear();
		for (Object o: hlNode.getOutgoing()){			
			DeleteHLArcCommand cmd=new DeleteHLArcCommand();
			cmd.setHLArc((HLArc)o);
			deleteHLArcCommands.add(cmd);
		}
		for (Object o: hlNode.getIncoming()){			
			DeleteHLArcCommand cmd=new DeleteHLArcCommand();
			cmd.setHLArc((HLArc)o);
			deleteHLArcCommands.add(cmd);
		}
	}
	

	/**
	 * Executes the command by first hiding all views which display the element
	 * that will be removed. Then the child gets removed from its parent. The
	 * container is stored for undoing purposes.
	 */
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.commands.Command#execute()
	 */
	@Override
	public void execute() {	
		 if (node instanceof HLPlace){
			 for (Object o: ((HLPlace)node).getTokens()){
				 for (Object o1: new Vector(((Token)o).eAdapters())){
					 if (o1 instanceof AbstractMultiViewerPage){						 
						 ((RONEditor)(((AbstractMultiViewerPage)o1).getEditor())).closeViewsShowing((Token)o);						 
					 }
				 }
			 }
		 }


		// Jetzt kann das Element aus dem Container gelscht werden
		
		parent = node.getRon();		
		createDeleteHLArcCommands(node);
		for(DeleteHLArcCommand cmd: deleteHLArcCommands)
			cmd.execute();
		node.setRon(null);
	}

	
	/**
	 * Undoes the command by readding the previously removed object to its
	 * former container that {@link #execute()} has saved.
	 */
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.commands.Command#undo()
	 */
	@Override
	public void undo() {
		node.setRon(parent);		
		for(DeleteHLArcCommand cmd: deleteHLArcCommands)
			cmd.undo();
	}

	/**
	 * Redoes the command by removing the object from its container again.
	 */
	@Override
	public void redo() {
		execute();
	}
}
