package roneditor.commands;

import org.eclipse.gef.commands.Command;

import ronmodel.NamedElement;

/**
 * Command to change the name of a NamedElement. This is undoable.
 * 
 * @author Tony Modica
 */
public class RenameNamedElementCommand extends Command {

	private static final String EditCommand_Label = "Rename Element";

	protected NamedElement namedElement;

	protected String oldName;

	protected String newName;

	public RenameNamedElementCommand(final NamedElement namedElement) {
		super(RenameNamedElementCommand.EditCommand_Label);
		this.namedElement = namedElement;
	}

	/**
	 * Executes the command by storing the old name of the named element and
	 * setting the new one.
	 */
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.commands.Command#execute()
	 */
	@Override
	public void execute() {
		oldName = namedElement.getName();
		namedElement.setName(newName);
	}

	/**
	 * Undoes the command by resetting the old name of the named element. The
	 * old name has been saved by {@link #execute()}.
	 */
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.commands.Command#undo()
	 */
	@Override
	public void undo() {
		namedElement.setName(oldName);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.commands.Command#redo()
	 */
	@Override
	public void redo() {
		namedElement.setName(newName);
	}

	/**
	 * @param newName
	 *            The newName to set.
	 */
	public void setNewName(final String newName) {
		this.newName = newName;
	}
}
