package roneditor.objectnetview;

import org.eclipse.gef.GraphicalViewer;
import org.eclipse.gef.editparts.ZoomManager;
import org.eclipse.gef.ui.actions.ActionBarContributor;
import org.eclipse.gef.ui.actions.DeleteRetargetAction;
import org.eclipse.gef.ui.actions.GEFActionConstants;
import org.eclipse.gef.ui.actions.RedoRetargetAction;
import org.eclipse.gef.ui.actions.UndoRetargetAction;
import org.eclipse.gef.ui.actions.ZoomComboContributionItem;
import org.eclipse.gef.ui.actions.ZoomInRetargetAction;
import org.eclipse.gef.ui.actions.ZoomOutRetargetAction;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.actions.ActionFactory;
import org.eclipse.ui.ide.IDEActionFactory;
import org.eclipse.ui.texteditor.ITextEditor;
import org.eclipse.ui.texteditor.ITextEditorActionConstants;

import roneditor.actions.AddTokenAction;
import roneditor.actions.AddTokenRetargetAction;
import roneditor.actions.FireTransitionAction;
import roneditor.actions.FireTransitionRetargetAction;
import roneditor.objectnetview.actions.DeleteBendPointAction;
import roneditor.objectnetview.actions.EditAction;
import roneditor.objectnetview.actions.EditRetargetAction;
import roneditor.ronview.actions.CompleteMappingConsistencyCheckAction;
import roneditor.ronview.actions.CompleteMappingConsistencyCheckRetargetAction;

/**
 * Constributor class to handle menue events.
 **/
public class ObjectNetEditorActionBarContributor_notUsed 
	extends ActionBarContributor	
{
	private IEditorPart activeEditorPart;	

	/**
	 * The {@link ZoomComboContributionItem} placed in the action bar.
	 */
	private ZoomComboContributionItem zoomComboContributionItem;

	/**
	 * Method for updating the {@link ZoomManager} of the
	 * {@link ZoomComboContributionItem} manually. This is especially needed if
	 * several {@link GraphicalViewer}s are used in a single page as the part
	 * service won't notice changes of the current viewer and does not notify
	 * the listening {@link ZoomComboContributionItem}.
	 */
	public void setZoomManager(final ZoomManager zoomManager) {
		zoomComboContributionItem.setZoomManager(zoomManager);
	}

	/**
	 * Creates a multi-page contributor.
	 */
	public ObjectNetEditorActionBarContributor_notUsed() {
		super();
	}

	/**
	 * Returns the action registed with the given text editor.
	 * @return IAction or null if editor is null.
	 */
	protected IAction getAction(ITextEditor editor, String actionID) {
		return (editor == null ? null : editor.getAction(actionID));
	}
	

	public void setActivePage(IEditorPart part) {
		if (activeEditorPart == part)
			return;

		activeEditorPart = part;

		IActionBars actionBars = getActionBars();
		if (actionBars != null) {

			ITextEditor editor =
				(part instanceof ITextEditor) ? (ITextEditor) part : null;

			actionBars.setGlobalActionHandler(
					ActionFactory.DELETE.getId(),
				getAction(editor, ITextEditorActionConstants.DELETE));
			actionBars.setGlobalActionHandler(
					ActionFactory.UNDO.getId(),
				getAction(editor, ITextEditorActionConstants.UNDO));
			actionBars.setGlobalActionHandler(
					ActionFactory.REDO.getId(),
				getAction(editor, ITextEditorActionConstants.REDO));
//			actionBars.setGlobalActionHandler(
//					ActionFactory.CUT.getId(),
//				getAction(editor, ITextEditorActionConstants.CUT));
//			actionBars.setGlobalActionHandler(
//					ActionFactory.COPY.getId(),
//				getAction(editor, ITextEditorActionConstants.COPY));
//			actionBars.setGlobalActionHandler(
//					ActionFactory.PASTE.getId(),
//				getAction(editor, ITextEditorActionConstants.PASTE));
			actionBars.setGlobalActionHandler(
					ActionFactory.SELECT_ALL.getId(),
				getAction(editor, ITextEditorActionConstants.SELECT_ALL));
			actionBars.setGlobalActionHandler(
				ActionFactory.FIND.getId(),
				getAction(editor, ITextEditorActionConstants.FIND));
			actionBars.setGlobalActionHandler(
					IDEActionFactory.BOOKMARK.getId(),
				getAction(editor, IDEActionFactory.BOOKMARK.getId()));
			actionBars.updateActionBars();
		}
	}
	
	
	/* (non-Javadoc)
	 * @see org.eclipse.gef.ui.actions.ActionBarContributor#buildActions()
	 */
	protected void buildActions()
	{
		addRetargetAction(new AddTokenRetargetAction());
		addRetargetAction(new EditRetargetAction());
		addRetargetAction(new FireTransitionRetargetAction());
		addRetargetAction(new CompleteMappingConsistencyCheckRetargetAction());


		addRetargetAction(new UndoRetargetAction());
		addRetargetAction(new RedoRetargetAction());
		addRetargetAction(new DeleteRetargetAction());
    
		addRetargetAction(new ZoomInRetargetAction());
		addRetargetAction(new ZoomOutRetargetAction());
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.ui.actions.ActionBarContributor#declareGlobalActionKeys()
	 */
	protected void declareGlobalActionKeys()
	{
		addGlobalActionKey(ActionFactory.PRINT.getId());
		addGlobalActionKey(ActionFactory.DELETE.getId());
	}
    
	/* (non-Javadoc)
	 * @see org.eclipse.ui.part.EditorActionBarContributor#contributeToMenu(org.eclipse.jface.action.IMenuManager)
	 */
	public void contributeToMenu(IMenuManager menuManager)
	{
		super.contributeToMenu(menuManager);

		// add a "View" menu after "Edit"
		MenuManager viewMenu = new MenuManager("View");
		viewMenu.add(getAction(GEFActionConstants.ZOOM_IN));
		viewMenu.add(getAction(GEFActionConstants.ZOOM_OUT));
        
		menuManager.insertAfter(IWorkbenchActionConstants.M_EDIT, viewMenu);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.part.EditorActionBarContributor#contributeToToolBar(org.eclipse.jface.action.IToolBarManager)
	 */
	public void contributeToToolBar(IToolBarManager toolBarManager)
	{
		super.contributeToToolBar(toolBarManager);

		toolBarManager.add(getAction(ActionFactory.UNDO.getId()));
		toolBarManager.add(getAction(ActionFactory.REDO.getId()));
    
		toolBarManager.add(new Separator());
		toolBarManager.add(getAction(ActionFactory.DELETE.getId()));

		toolBarManager.add(new Separator());
		toolBarManager.add(getAction(GEFActionConstants.ZOOM_IN));
		toolBarManager.add(getAction(GEFActionConstants.ZOOM_OUT));

		final String[] zoomStrings = new String[] { ZoomManager.FIT_ALL,
				ZoomManager.FIT_HEIGHT, ZoomManager.FIT_WIDTH };

		zoomComboContributionItem = new ZoomComboContributionItem(getPage(),
				zoomStrings);
		toolBarManager.add(zoomComboContributionItem);
		
		toolBarManager.add(new Separator());		
		toolBarManager.add(getAction(AddTokenAction.ID));
		toolBarManager.add(getAction(EditAction.ID));
		toolBarManager.add(getAction(FireTransitionAction.ID));	
		toolBarManager.add(getAction(DeleteBendPointAction.ID));
		toolBarManager.add(new Separator());
		toolBarManager.add(getAction(CompleteMappingConsistencyCheckAction.ID));

		//toolBarManager.add(getAction(AutoLayoutAction.ID));
	}
}
