/**
 * 
 */
package roneditor.objectnetview.actions;

import java.util.List;

import org.eclipse.gef.EditPart;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.commands.CompoundCommand;
import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.ui.IEditorPart;

import roneditor.objectnetview.commands.AutoLayoutCommand;
import roneditor.objectnetview.commands.DeleteBendpointCommand;
import ronmodel.BendPoint;
import ronmodel.LHS;
import ronmodel.NAC;
import ronmodel.ONArc;
import ronmodel.ONNode;
import ronmodel.ONPlace;
import ronmodel.ONTransition;
import ronmodel.ObjectNet;
import ronmodel.RHS;

/**
 * Action die das ausgewhlte Object-Netz layoutet.
 * 
 * @author Mario
 */
public class AutoLayoutAction extends SelectionAction {

	static public String ID = "Auto_Layout_Action";
	
	private ObjectNet objNet= null;


	/**
	 * Initialisiert die Action. Setzt ID, Description und Name der Action.
	 * 
	 * @param part EditorPart zu dem diese Action gehrt.
	 */
	public AutoLayoutAction(IEditorPart part) {
		super(part);
		setId(ID);
		setDescription("AutoLayout");
		setText("AutoLayout");
	}
	
	
	
	/**
	 * Diese Funktion aurufen um die Action auszufhren.
	 * Es wird ein Command erzeugt und im entsprechenden EditorPart ausgefhrt.
	 * 
	 */
	public void run() {
		execute(getCommand());
	}
		
	/**
	 * Baut ein CompoundCommand zusammen, welches die ONNodes verschiebt und alle Bendpoints lscht.
	 * 
	 * @return Command welches die ONNodes verschiebt und alle Bendpoints lscht
	 */
	public Command getCommand(){
		// ObjectNetz aus der Selektion holen
		List selections= this.getSelectedObjects();
		if (selections == null || selections.isEmpty()) {
			return null;
		}
		EditPart selected = (EditPart)selections.get(0);
		if (selected != null) {
			Object model = selected.getModel();
			// Object-Netz direkt angewhlt
			if (model instanceof ObjectNet) {
				objNet = (ObjectNet)model;
			// ONNode angewhlt, Object-Netz damit indirekt bestimmen
			} else if (model instanceof ONNode) {
				objNet = ((ONNode)model).getObjectNet();
			// ONArc angewhlt, Object-Netz damit indirekt bestimmen
			} else if (model instanceof ONArc) {
				objNet = ((ONArc)model).getObjectNet();
			} else {
				return null;
			}
		} else { 
			return null;
		}		
		CompoundCommand cCommand = new CompoundCommand();
		// ONNodes verschieben
		AutoLayoutCommand autoLayoutCommand = new AutoLayoutCommand();			
		autoLayoutCommand.setObjectNet(objNet);
		cCommand.add(autoLayoutCommand);
		// alle BendPoints lschen
		List<ONArc> arcs=objNet.getArcs();
		for (ONArc arc: arcs) {
			List<BendPoint> bendPoints= arc.getBendpoints();
			for (BendPoint bendPoint:bendPoints) {
				DeleteBendpointCommand deleteBendPointCommand = new DeleteBendpointCommand();
				deleteBendPointCommand.setONArc(arc);
				cCommand.add(deleteBendPointCommand);
			}
		}
		return cCommand; 
	}



	/**
	 * Gibt zurck ob die action ausfhrbar ist.
	 * 
	 * @return true, wenn ein ObjectNet,ONNode oder ONArc, dass erste Objekt in der Liste der selektierten Objekte ist, anderfalls false 
	 */
	@Override
	protected boolean calculateEnabled() {
		List selections= this.getSelectedObjects();
		if (selections == null || selections.isEmpty()) {
			return false;
		}
		EditPart selected = (EditPart)selections.get(0);
		if (selected != null) {
			Object model = selected.getModel();
			if ((model instanceof ObjectNet) 
				|| (model instanceof ONNode)
				|| (model instanceof ONArc)) {
					return true;
			}
		}
		return false;
				
	}
	
	


}
