/**
 * 
 */
package roneditor.objectnetview.commands;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.draw2d.graph.DirectedGraph;
import org.eclipse.draw2d.graph.DirectedGraphLayout;
import org.eclipse.draw2d.graph.Edge;
import org.eclipse.draw2d.graph.Node;
import org.eclipse.gef.commands.Command;

import ronmodel.ONArc;
import ronmodel.ONNode;
import ronmodel.ObjectNet;

/**
 * Command das die ONNodes mittels DirectedGraphLayout indirekt verschiebt.
 * 
 * @author Mario
 */
public class AutoLayoutCommand extends Command {

	private ObjectNet objectNet= null;
	private Map<ONNode,Integer> oldX, oldY, newX, newY;
	private Map<ONArc,Integer> oldAnchorNr_S, oldAnchorNr_T;
	

	public AutoLayoutCommand() {
		setLabel("AutoLayout");
		oldX = new HashMap<ONNode,Integer>();
		oldY = new HashMap<ONNode,Integer>();
		newX = new HashMap<ONNode,Integer>();
		newY = new HashMap<ONNode,Integer>();
		oldAnchorNr_S = new HashMap<ONArc,Integer>();
		oldAnchorNr_T = new HashMap<ONArc,Integer>();
	}
	
	
	public void execute() {
		// ObjectNetz in einen DirectedGraph transformieren
		DirectedGraph dg = new DirectedGraph();
		// erst alle Knoten
		for (ONNode onNode:((List<ONNode>)objectNet.getNodes())) {
			Node node = new Node();
			node.data= onNode;
			node.x = onNode.getX();
			node.y = onNode.getY();
			// Knoten einfgen
			dg.nodes.add(node);
			oldX.put(onNode, node.x);
			oldY.put(onNode, node.y);
		}
		// dann alle Kanten
		for  (ONArc onArc:((List<ONArc>)objectNet.getArcs())) {
			ONNode onSource = onArc.getSource();
			ONNode onTarget = onArc.getTarget();
			Node source= null;
			Node target= null;
			// entsprechenden Source und TargetKnoten finden
			for (int i = 0; i < dg.nodes.size(); i++) {
				if (dg.nodes.getNode(i).data == onSource) {
					source = dg.nodes.getNode(i);
				}
				if (dg.nodes.getNode(i).data == onTarget) {
					target = dg.nodes.getNode(i);
				}
			}
			Edge edge = new Edge(source,target);
			edge.data = onArc;
			// Kante einfgen
			dg.edges.add(edge);
			oldAnchorNr_S.put(onArc, onArc.getAnchorNrS());
			oldAnchorNr_T.put(onArc, onArc.getAnchorNrT());
		}
		DirectedGraphLayout dgl = new DirectedGraphLayout();
		dgl.visit(dg);

		
		// ins eigentliche Modell reinschreiben
		for (int i= 0; i < dg.nodes.size() ; i++) {
			ONNode onNode = (ONNode)dg.nodes.getNode(i).data;
			Integer newXvalue = dg.nodes.getNode(i).x;
			// +50 da sonst oben der Name des ObjectNetzes strt
			Integer newYvalue = dg.nodes.getNode(i).y+50;
			(onNode).setX(newXvalue);
			(onNode).setY(newYvalue);
			newX.put(onNode, newXvalue);
			newY.put(onNode, newYvalue);
			
		}		
		// Anchors auf -1 setzen
		for (ONArc onArc:(List<ONArc>)objectNet.getArcs()) {
			onArc.setAnchorNrS(-1);
			onArc.setAnchorNrT(-1);
		}
		
		
	}
	
	/**
	 * Macht die nderungen rckgngig, indem die alten gemerkten Werte zurckgeschrieben werden.
	 */
	public void undo() {
		for (ONNode onNode:(List<ONNode>)objectNet.getNodes()) {
			onNode.setX(oldX.get(onNode));
			onNode.setY(oldY.get(onNode));
		}
		for (ONArc onArc:(List<ONArc>)objectNet.getArcs()) {
			onArc.setAnchorNrS(oldAnchorNr_S.get(onArc));
			onArc.setAnchorNrT(oldAnchorNr_T.get(onArc));
		}
	}

	/**
	 * Wiederholt die nderungen, indem die neuen gemerkten Werte wieder reingeschrieben werden.
	 */
	public void redo() {
		for (ONNode onNode:(List<ONNode>)objectNet.getNodes()) {
			onNode.setX(newX.get(onNode));
			onNode.setY(newY.get(onNode));
		}		
		// Anchors auf -1 setzen
		for (ONArc onArc:(List<ONArc>)objectNet.getArcs()) {
			onArc.setAnchorNrS(-1);
			onArc.setAnchorNrT(-1);
		}
	}	
	
	/**
	 * True if the command can be executed, false otherwise.
	 */
	public boolean canExecute() {
		if (objectNet == null)
			return false;
		else 
			return true;
	}
	
	
	
	/**
	 * @param objectNet the objectNet to set
	 */
	public void setObjectNet(ObjectNet objectNet) {
		this.objectNet = objectNet;
	}
	
	
}
