package roneditor.objectnetview.commands;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.gef.commands.Command;

import ronmodel.Codomain;
import ronmodel.LHS;
import ronmodel.Mapping;
import ronmodel.ONTransition;
import ronmodel.ObjectNet;


/**
 * Command class for a ONPlace Mapping.
 * Connects a ONPlace with a ONPlace via a Mapping.
 */
public class MappingONTransitionCommand extends Command {	
	private static final String ConnectionCommand_Description = 
		"Mapping: ONTransition mapping command";

	protected ONTransition source;
	protected ONTransition target;
	protected Mapping mapping;
	protected Integer mappingNr;
	
	/**
	 * Constructor.
	 */
	public MappingONTransitionCommand() {
		super(ConnectionCommand_Description);
	}
	
	
	/**
	 * Entry point for the command execution.
	 */
	public void execute() { 
		// Reihenfolge im Moment sehr wichtig
		// sobald Codomain gendert wird, wird Notifier geschickt, was dann updatet

		ObjectNet on = target.getObjectNet();
		mapping.setSource(source);
		mapping.setTarget(target);
		mapping.setCodomain((Codomain)on);
		// kann erst gemacht werden wenn das Mapping komplett ist
		this.calculateMappingNumber();
		mapping.setMappingNr(mappingNr);
		// Source und Target nochmal neu setzen um notifyChanged der entsprechenden ONTransition auszulsen, 
		// da erst jetzt die MappingNummer bestimmt ist
		mapping.setSource(null);
		mapping.setTarget(null);
		mapping.setSource(source);
		mapping.setTarget(target);
		
//		TEMP
//		MappingConsistencyResult mcr=MappingConsistencyCheck.checkMappingConsistency(mapping.getCodomain());
//		System.out.println(mcr.getMessage());
//		END TEMP
	}
	
	/**
	 * Undo command execution.
	 */
	public void undo() {
		// Reihenfolge im Moment sehr wichtig
		// sobald Codomain gendert wird, wird Notifier geschickt, was dann updatet
		mapping.setCodomain(null);
		mapping.setMappingNr(-1);
		// Mapping aus der MappingNr verwaltenen Liste rausnehmen
		((LHS)source.getObjectNet()).getRule().getMappingNumberList().removeMapping(mapping);
		mapping.setSource(null);
		mapping.setTarget(null);
	}
	
	/**
	 * Redo command execution.
	 */
	public void redo() {
		execute();
	}

	/**
	 * True if the command can be executed, false otherwise.
	 */
	public boolean canExecute() {
		if (source == null || target == null || mapping == null || source == target)
			return false;
		else return true;
	}

	/**
	 * Returns the command label.
	 */
	public String getLabel() {
		return ConnectionCommand_Description;
	}

	public void setSource(ONTransition source) {
		this.source = source;
	}

	public void setTarget(ONTransition target) {
		this.target = target;
	}

	/**
	 * @return the source
	 */
	public ONTransition getSource() {
		return source;
	}

	/**
	 * @return the target
	 */
	public ONTransition getTarget() {
		return target;
	}


	/**
	 * @return the mapping
	 */
	public Mapping getMapping() {
		return mapping;
	}


	/**
	 * @param mapping the mapping to set
	 */
	public void setMapping(Mapping mapping) {
		this.mapping = mapping;
	}

	/**
	 * Calculate and Sets the MappingNumber.
	 */
	protected void calculateMappingNumber() {
		// die Mapping die diesen ONNode als Source haben
		// inklusive diesem neuen, welches mit diesem Command hinzugefgt werden soll
		List mappings= source.getOutgoingM();
		List<Mapping> tempMappings = new ArrayList<Mapping>();
		tempMappings.addAll(mappings);		
		mappingNr = ((LHS)source.getObjectNet()).getRule().getMappingNumberList().getMappingNumber(tempMappings)+1;
	}
}
