/**
 * 
 */
package roneditor.objectnetview.edit.parts;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.PolylineConnection;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.gef.DragTracker;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.Request;
import org.eclipse.gef.editparts.AbstractGraphicalEditPart;

import roneditor.objectnetview.edit.policies.ONArcLabelMovePolicy;
import roneditor.objectnetview.figures.ONArcLabelFigure;
import roneditor.objectnetview.figures.ONArcLabelMidPointLocator;
import ronmodel.ONArc;
import ronmodel.RonmodelPackage;

public class ONArcLabelEditPart extends AbstractGraphicalEditPart implements
		Adapter {

	

	private Notifier target;

	
	

	/**
	 * Bekommt eine Pseudo Modell (ein einfaches Object, ber ObjectID eindeutig und nur darum gehts dabei) 
	 * und merkt ihn sich als sein Model.
	 * Dies ist beim ONArcLabel mglich,da man ihn direkt als Child von ONArcEditPart einhngen kann, da hier 
	 * das Label auch angezeigt wird, wenn er sich ausserhalb der ParentFigure befindet.
	 * @param obj Pseudo-Modelobjekt fr diesen EditPart
	 */
	public ONArcLabelEditPart(final Object obj) {
		setModel(obj);
	}
	
	
	/** 
	 * Erstellt die Figure (ein Label mit Hintergrund-Rectangle) fr diesen EditPart und setzt den Text entsprechend des Models.
	 */
	@Override
	protected IFigure createFigure() {
		ONArcLabelFigure figure = new ONArcLabelFigure();
		Integer weight = getCastedParentModel().getWeight();
		if (weight > 1) {
			figure.setWeight(weight.toString());
		} else {
			figure.setWeight("");
		}
		return figure;
	}

	
	
	/* (non-Javadoc)
	 * @see org.eclipse.emf.common.notify.Adapter#getTarget()
	 */
	public Notifier getTarget() {
		return target;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.emf.common.notify.Adapter#isAdapterForType(java.lang.Object)
	 */
	public boolean isAdapterForType(Object type) {
		return type.equals(getModel().getClass());
	}


	/* (non-Javadoc)
	 * @see org.eclipse.emf.common.notify.Adapter#setTarget(org.eclipse.emf.common.notify.Notifier)
	 */
	public void setTarget(Notifier newTarget) {
		target= newTarget;
	}

	
	/**
	 * Holt das eigentliche Model den passnden ONArc
	 * 
	 * @return ONArc Modelobjekt fr diesen EditPart
	 */
	public ONArc getCastedParentModel() {
		return (ONArc)getParent().getModel();
	}
	
	
	

	
	/** 
	 * Setzt den Text und die Position entsprechend des ModelObjektes neu.
	 */
	protected void refreshVisuals()
	{
		
		final Point offset = new Point(getCastedParentModel().getWeightLabelX(),
				getCastedParentModel().getWeightLabelY());
		
		final ONArcLabelFigure figure = (ONArcLabelFigure) getFigure();

		Integer weight = getCastedParentModel().getWeight();
		String labelText = "";
		if (weight > 1) {
			labelText = weight.toString();
		}

		figure.setWeight(labelText);
		final ONArcEditPart parent = (ONArcEditPart) getParent();
		final PolylineConnection connFigure = (PolylineConnection) parent.getFigure();
		final ONArcLabelMidPointLocator constraint = new ONArcLabelMidPointLocator(
				labelText, offset, connFigure);
		parent.setLayoutConstraint(this, getFigure(), constraint);
		
	}
	
		
	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
	 */
	@Override
	protected void createEditPolicies() {
		installEditPolicy(EditPolicy.GRAPHICAL_NODE_ROLE,
				new ONArcLabelMovePolicy());
	}

	/**
	 * @see org.eclipse.gef.EditPart#getDragTracker(org.eclipse.gef.Request)
	 */
	@Override
	public DragTracker getDragTracker(final Request request) {
		return new DragONArcLabelTracker(this, (ONArcEditPart) getParent());
	}



	
	public void notifyChanged(final Notification notification) {
		final int featureId = notification.getFeatureID(RonmodelPackage.class);

		if (notification.getNotifier() instanceof ONArc) {
			switch (featureId) {
			case RonmodelPackage.ON_ARC__WEIGHT:
			case RonmodelPackage.ON_ARC__WEIGHT_LABEL_X:
			case RonmodelPackage.ON_ARC__WEIGHT_LABEL_Y:
				refreshVisuals();
				break;
			default:
			}
		}
	}
	
	@Override
	public void activate() {
		if (isActive()) {
			return;
		}

		// start listening for changes in the model
		getCastedParentModel().eAdapters().add(this);
		super.activate();

		refreshVisuals();
	}

	@Override
	public void deactivate() {
		if (!isActive()) {
			return;
		}

		// stop listening for changes in the model
		getCastedParentModel().eAdapters().remove(this);
		super.deactivate();
	}
}
