/**
 * 
 */
package roneditor.objectnetview.edit.policies;


import java.util.List;

import org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy;

import roneditor.objectnetview.commands.AbstractConnectionCommand;
import roneditor.objectnetview.commands.MappingONPlaceCommand;
import roneditor.objectnetview.commands.MappingONTransitionCommand;
import roneditor.objectnetview.commands.ReconnectONArcCommand;
import ronmodel.Codomain;
import ronmodel.Mapping;
import ronmodel.ONArc;
import ronmodel.ONNode;

/**
 * @author Alex
 *
 */
public abstract class ONNodeEditPolicy extends GraphicalNodeEditPolicy {

	
	/**
	 * Checks if there already exist a mapping between source and target.
	 * @param source the source
	 * @param target the target
	 * @return the Mapping between source and target or null if nothing found.
	 */
	private static Mapping thereIsAlreadyMapping(ONNode source, ONNode target) {
		if (source==null || target==null) return null;
		if (target.getObjectNet() instanceof Codomain) {
			List mappings= ((Codomain)target.getObjectNet()).getMappings();
			for (Object o:mappings) {
				Mapping m = (Mapping)o;
				if (m.getSource() == source &&  m.getTarget() == target) {
					return m;
				}
			}
		} 			
		return null;	
	}
	
	/**
	 * Checks if there already exist a mapping between source and target.
	 * @param command MappingONPlaceCommand
	 * @return the Mapping between source and target or null if nothing found.
	 */
	protected static Mapping thereIsAlreadyMapping(MappingONPlaceCommand command) {
		ONNode source=command.getSource();
		ONNode target=command.getTarget();
		return thereIsAlreadyMapping(source, target);
	}

	/**
	 * Checks if there already exist a mapping between source and target.
	 * @param command MappingONPlaceCommand
	 * @return the Mapping between source and target or null if nothing found.
	 */
	protected static Mapping thereIsAlreadyMapping(MappingONTransitionCommand command) {
		ONNode source=command.getSource();
		ONNode target=command.getTarget();
		return thereIsAlreadyMapping(source, target);
	}	
	
	
	
	
	
	
	/**
	 * Checks if there already exist a mapping between target and LHS.
	 * @param source the source
	 * @param target the target
	 * @return the Mapping that allready exists or null
	 */
	private static Mapping thereIsAlreadyMappingFromLHS(ONNode source, ONNode target) {
		if (source==null || target==null) return null;
		if (target.getObjectNet() instanceof Codomain) {
			List mappings= ((Codomain)target.getObjectNet()).getMappings();
			for (Object o:mappings) {
				Mapping m = (Mapping)o;
				if (m.getTarget() == target) {
					return m;
				}
			}
		}
		return null;	
	}
	
	/**
	 * Checks if there already exist a mapping between target and LHS.
	 * @param command MappingONPlaceCommand
	 * @return the Mapping that allready exists or null
	 */
	protected static Mapping thereIsAlreadyMappingFromLHS(MappingONPlaceCommand command) {
		ONNode source=command.getSource();
		ONNode target=command.getTarget();
		return thereIsAlreadyMappingFromLHS(source, target);
	}

	/**
	 * Checks if there already exist a mapping between target and LHS.
	 * @param command MappingONTransitionCommand
	 * @return the Mapping that allready exists or null
	 */
	protected static Mapping thereIsAlreadyMappingFromLHS(MappingONTransitionCommand command) {
		ONNode source=command.getSource();
		ONNode target=command.getTarget();
		return thereIsAlreadyMappingFromLHS(source, target);
	}	
	
	
	
	
	
	
	
	
	/**
	 * Checks if there already exist a mapping between source and the Side(RHS,NAC1,NAC2,...) of the Target.
	 * @param source the source
	 * @param target the target
	 * @return the Mapping that allready exists or null
	 */
	private static Mapping thereIsAlreadyMappingToSide(ONNode source, ONNode target) {
		if (source==null || target==null) return null;
		if (target.getObjectNet() instanceof Codomain) {
			List mappings= ((Codomain)target.getObjectNet()).getMappings();
			for (Object o:mappings) {
				Mapping m = (Mapping)o;
				if (m.getSource() == source) {
					return m;
				}
			}
		}
		return null;	
	}
	
	/**
	 * Checks if there already exist a mapping between source and the Side(RHS,NAC1,NAC2,...) of the Target.
	 * @param command MappingONPlaceCommand
	 * @return the Mapping that allready exists or null
	 */
	protected static Mapping thereIsAlreadyMappingToSide(MappingONPlaceCommand command) {
		ONNode source=command.getSource();
		ONNode target=command.getTarget();
		return thereIsAlreadyMappingToSide(source, target);
	}

	/**
	 * Checks if there already exist a mapping between source and the Side(RHS,NAC1,NAC2,...) of the Target.
	 * @param command MappingONTransitionCommand
	 * @return the Mapping that allready exists or null
	 */
	protected static Mapping thereIsAlreadyMappingToSide(MappingONTransitionCommand command) {
		ONNode source=command.getSource();
		ONNode target=command.getTarget();
		return thereIsAlreadyMappingToSide(source, target);
	}	
	
	
	
	/**
	 * Checks if there already exist a connection between source and target.
	 * @param onArc the onArc to compare with (ReconnectCommand) or null (otherwise)
	 * @param source the source
	 * @param target the target
	 * @return true if there is already an connection between source and target, false otherwise
	 */
	private static boolean thereIsAlreadyONArc(ONArc onArc, ONNode source, ONNode target) {
		if (source==null || target==null) return false;
		for (Object o:source.getOutgoing()){
			ONArc arc=((ONArc)o); 
			if (arc.getTarget()==target && arc!=onArc) return true;			
		}
		return false;		
	}
	
	/**
	 * Checks if there already exist a connection between source and target.
	 * @param command AbstractConnectionCommand
	 * @return true if there is already an connection between source and target, false otherwise
	 */
	protected static boolean thereIsAlreadyONArc(AbstractConnectionCommand command) {
		ONNode source=command.getSource();
		ONNode target=command.getTarget();
		return thereIsAlreadyONArc(null,source, target);
	}

	/**
	 * Checks if there already exist a connection between source and target.
	 * @param command ReconnectONArcCommand
	 * @return true if there is already an connection between source and target, false otherwise
	 */
	protected static boolean thereIsAlreadyONArc(ReconnectONArcCommand command) {
		ONArc onArc=command.getONArc();
		ONNode source=command.getNewSource();
		ONNode target=command.getNewTarget();
		if (source==null){
			source=command.getONArc().getSource();
		} 
		if (target==null){
			target=command.getONArc().getTarget();
		}
		return thereIsAlreadyONArc(onArc, source, target);
	}
	
	/**
	 * Checks if the command changes something (important for Undo / Redo)
	 * @param command ReconnectONArcCommand
	 * @return true if the command changes something, false otherwise
	 */
	protected static boolean nothingChanged(ReconnectONArcCommand command){
		ONNode oldSource=command.getOldSource();
		ONNode oldTarget=command.getOldTarget();
		ONNode newSource=command.getNewSource();
		ONNode newTarget=command.getNewTarget();
		int newAnchorNrS=command.getNewAnchorNrS();
		int newAnchorNrT=command.getNewAnchorNrT();
		if (newSource==null){
			newSource=command.getONArc().getSource();
			newAnchorNrS=command.getONArc().getAnchorNrS();
		} 
		if (newTarget==null){
			newTarget=command.getONArc().getTarget();
			newAnchorNrT=command.getONArc().getAnchorNrT();
		}
		if (newSource!=oldSource) return false;
		if (newTarget!=oldTarget) return false;
		if (newAnchorNrS!=command.getOldAnchorNrS()) return false;
		if (newAnchorNrT!=command.getOldAnchorNrT()) return false;
		return true;
	}

}
