package roneditor.objectnetview.edit.utils;

import org.eclipse.core.runtime.Assert;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPartFactory;

import roneditor.edit.parts.LabelEditPart;
import roneditor.objectnetview.edit.parts.ONArcEditPart;
import roneditor.objectnetview.edit.parts.ONArcLabelEditPart;
import roneditor.objectnetview.edit.parts.ONPlaceEditPart;
import roneditor.objectnetview.edit.parts.ONTransitionEditPart;
import roneditor.objectnetview.edit.parts.ObjectNetEditPart;
import ronmodel.ONArc;
import ronmodel.ONPlace;
import ronmodel.ONTransition;
import ronmodel.ObjectNet;

/**
 * Factory class for edit parts. Here, the corresponding edit parts of a the
 * model elements are created.
 * 
 * @author Tony Modica
 */
public class ObjectNetViewEditPartFactory implements EditPartFactory {

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.EditPartFactory#createEditPart(org.eclipse.gef.EditPart,
	 *      java.lang.Object)
	 */
	public EditPart createEditPart(final EditPart context, final Object obj) {
		// HINT Passt diese Methode an Euer Modell und Eure EditParts an
		if (obj == null) {
			return null;
		} if(obj instanceof ObjectNet){
			ObjectNetEditPart p =  new ObjectNetEditPart((ObjectNet)obj);
			return p;
		}else if(obj instanceof ONTransition){
			 ONTransitionEditPart p = new ONTransitionEditPart((ONTransition)obj);
			 return p;
		}else if(obj instanceof ONPlace){
			ONPlaceEditPart p = new ONPlaceEditPart((ONPlace)obj);
			return p;
		}else if (obj instanceof ONArc){
			return new ONArcEditPart((ONArc)obj);
		} else if (context instanceof ObjectNetEditPart) {
			return new LabelEditPart(obj);
		} else if (context instanceof ONArcEditPart) {
			// ONArcLabelEditParts do not have a significant model
			return new ONArcLabelEditPart(obj);
		}
		// Fehler melden
		Assert
				.isTrue(
						false,
						"ObjectNetViewEditPartFactory could not create an EditPart for the model element of type "
								+ obj.getClass());
		return null;
	}
}
