package roneditor.objectnetview.figures;

import org.eclipse.draw2d.Connection;
import org.eclipse.draw2d.MidpointLocator;
import org.eclipse.draw2d.PolylineConnection;
import org.eclipse.draw2d.geometry.Point;

public class ONArcLabelMidPointLocator extends MidpointLocator {

	String text;

	Point offset;

	public ONArcLabelMidPointLocator(final String text, final Point offset,
			final PolylineConnection arcFigure) {
		super(arcFigure, 0);
		this.text = text;
		this.offset = offset;
	}

	/**
	 * Returns the point of reference associated with this locator. This point
	 * will be midway between points at 'point[size/2-1]' and 'point[size/2]'
	 * and shifted by the coordinate for the Arc's Label given by the model.
	 * 
	 * @return the reference point
	 */
	@Override
	protected Point getReferencePoint() {
		final Connection conn = getConnection();
		Point p = Point.SINGLETON;
		final int pointsSize = conn.getPoints().size();
		if (pointsSize % 2 != 0) {
			p = conn.getPoints().getPoint(pointsSize / 2);
		} else {
			final Point p1 = conn.getPoints().getPoint(pointsSize / 2 - 1);
			final Point p2 = conn.getPoints().getPoint(pointsSize / 2);
			conn.translateToAbsolute(p1);
			conn.translateToAbsolute(p2);
			p.x = (p2.x - p1.x) / 2 + p1.x;
			p.y = (p2.y - p1.y) / 2 + p1.y;
		}
		return p.translate(offset.x, offset.y);
	}

}