package roneditor.objectnetview.figures;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.ConnectionAnchor;
import org.eclipse.draw2d.Ellipse;
import org.eclipse.draw2d.EllipseAnchor;
import org.eclipse.draw2d.FlowLayout;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.RectangleFigure;
import org.eclipse.draw2d.StackLayout;
import org.eclipse.draw2d.XYLayout;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.swt.graphics.Color;

import roneditor.objectnetview.anchors.ONAnchor;

/**
 * Represents the graphical layout of a transition.
 */
public class ONPlaceFigure 
	extends AbstractONNodeFigure
{
	private final int tokenSize = 6; //size of the tokens (6)
	private final int hugeTokenSize = 20; //size of the huge tokens (if more than 4) (20)
	private final int tokenPad = 4; //space between tokens (4)
	static public final int eSize = 30; //size of the place-ellipse (30) - NICHT kleiner als 20!!	
	private final Color tokenColor = ColorConstants.black; //color of the tokens
	private final Color tokenLabelColor = ColorConstants.white; //color of the text (tokencount)



	protected EllipseAnchor incomingCBConnectionAnchor;
	protected EllipseAnchor outgoingCBConnectionAnchor;
	protected FlowLayout layout;
	protected List<ONAnchor> incomingConnectionAnchors = new ArrayList<ONAnchor>();
	protected List<ONAnchor> outgoingConnectionAnchors = new ArrayList<ONAnchor>();
	private int tokens=0;
	private EllipseFigure e;
	private RectangleFigure tokenFigure;
	
	public ONPlaceFigure() {		
		setLayoutManager(new XYLayout());
		setOutline(false);
		setFill(false);	
		
		e = new EllipseFigure();		
		e.setSize(eSize, eSize);
		e.setBackgroundColor(ColorConstants.white);
		e.setOpaque(false);
		incomingCBConnectionAnchor = new EllipseAnchor(e);
		outgoingCBConnectionAnchor = new EllipseAnchor(e);
		createAnchors();		
		this.add(e);
		mappingLabel = new Label("");
		mappingLabel.setForegroundColor(ColorConstants.blue);
		add(mappingLabel);
		
//		label = new Label("");
//		label.setBackgroundColor(ColorConstants.white);
//		label.setOpaque(true);		
//		this.add(label);
		
		tokenFigure = new RectangleFigure();
		tokenFigure.setLayoutManager(new XYLayout());
		tokenFigure.setOutline(false);
		tokenFigure.setMaximumSize(new Dimension(eSize-3,eSize-3));
		this.add(tokenFigure);
	}
	
//	public void setName(String name){
//		label.setText(name);
//		validate();
//	}
	
	/**
	 * Puts one token on the place.
	 */
	private void addOneToken() {
		final Ellipse token1 = new Ellipse();
		token1.setSize(tokenSize, tokenSize);
		token1.setBackgroundColor(tokenColor);

		tokenFigure.add(token1);

		tokenFigure.getLayoutManager().setConstraint(token1,
				new Rectangle(0, 0, -1, -1));
	}

	/**
	 * Puts two tokens on the place.
	 */
	private void addTwoTokens() {
		final Ellipse token1 = new Ellipse();
		token1.setSize(tokenSize, tokenSize);
		token1.setBackgroundColor(tokenColor);
		tokenFigure.add(token1);

		final Ellipse token2 = new Ellipse();
		token2.setSize(tokenSize, tokenSize);
		token2.setBackgroundColor(tokenColor);
		tokenFigure.add(token2);

		tokenFigure.getLayoutManager().setConstraint(token1,
				new Rectangle(0, 0, -1, -1));
		tokenFigure.getLayoutManager().setConstraint(token2,
				new Rectangle(tokenSize + tokenPad, 0, -1, -1));
	}

	/**
	 * Puts three tokens on the place.
	 */
	private void addThreeTokens() {
		final Ellipse token1 = new Ellipse();
		token1.setSize(tokenSize, tokenSize);
		token1.setBackgroundColor(tokenColor);
		tokenFigure.add(token1);

		final Ellipse token2 = new Ellipse();
		token2.setSize(tokenSize, tokenSize);
		token2.setBackgroundColor(tokenColor);
		tokenFigure.add(token2);

		final Ellipse token3 = new Ellipse();
		token3.setSize(tokenSize, tokenSize);
		token3.setBackgroundColor(tokenColor);
		tokenFigure.add(token3);

		tokenFigure.getLayoutManager().setConstraint(token1,
				new Rectangle(0, 0, -1, -1));
		tokenFigure.getLayoutManager().setConstraint(token2,
				new Rectangle(tokenSize + tokenPad, 0, -1, -1));
		tokenFigure.getLayoutManager().setConstraint(
				token3,
				new Rectangle((tokenSize + tokenPad) / 2,
						(tokenSize + tokenPad) / 2, -1, -1));
	}

	
	/**
	 * Sets the tokens onto the place
	 * @param tokens token count
	 */
	public void setTokens(int tokens){
		this.tokens=tokens;
		tokenFigure.removeAll();
		switch (tokens) {
		case 0:
			break;
		case 1:
			addOneToken();
			break;
		case 2:
			addTwoTokens();
			break;
		case 3:
			addThreeTokens();
			break;
		default:
			String str=String.valueOf(tokens);
			final Ellipse token = new Ellipse();
			token.setSize(hugeTokenSize,hugeTokenSize);
			token.setBackgroundColor(tokenColor);			
			tokenFigure.add(token);
			
			tokenFigure.getLayoutManager().setConstraint(token,
					new Rectangle(0, 0, -1, -1));

			final Label tokenLabel = new Label(str);			
			tokenLabel.setForegroundColor(tokenLabelColor);
			tokenFigure.add(tokenLabel);
			
			if (tokenLabel.getTextBounds().width+4>hugeTokenSize){
				//set tooltip and marks the label with ".."
				this.setToolTip(new Label("Tokenanzahl: "+ str));
				while(tokenLabel.getText().length()>2 && tokenLabel.getTextBounds().width+4>hugeTokenSize){
					tokenLabel.setText(tokenLabel.getText().substring(0, tokenLabel.getText().length()-1));
				}
				tokenLabel.setText(tokenLabel.getText().substring(0, tokenLabel.getText().length()-2));
				tokenLabel.setText(tokenLabel.getText()+"..");
			} else //no tooltip
				this.setToolTip(null);

			tokenFigure.getLayoutManager().setConstraint(tokenLabel,
					new Rectangle((hugeTokenSize-tokenLabel.getTextBounds().width)/2, (hugeTokenSize-tokenLabel.getTextBounds().height)/2, -1, -1));
						
			break;

		}
	}

	/** 
	 * Returns the tokens
	 *
	 * @return  tokens
	 */
	public int getTokens(){
		return tokens;
	}
	
	@Override
	public void validate(){		
//		// Berhrt das Label oder die Ellipse den linken Rand der Figure?
//		int labelEllipseDiff = label.getPreferredSize().width-eSize;
//		
//		Rectangle r = new Rectangle(label.getTextBounds());		
//		label.setBounds(r);
		
		Dimension d = getPreferredSize(-1, -1);
		
		this.setMaximumSize(d);
		this.setMinimumSize(d);
		this.setPreferredSize(d);
		this.setOutline(false);

//		if (labelEllipseDiff < 0) {
			// Ellipse ist breiter als das Label!

			// Die Ellipse fllt die Figure aus
			getLayoutManager().setConstraint(e, new Rectangle(0, 0,
					eSize, eSize));


			getLayoutManager().setConstraint(mappingLabel, new Rectangle(eSize+3,1,-1,-1));
			//place label (name)
//			getLayoutManager().setConstraint(label, new Rectangle(
//					-labelEllipseDiff / 2, eSize + 3, -1, -1));
			// tokenLabel steht mittig in der Ellipse
			getLayoutManager().setConstraint(tokenFigure,
					new Rectangle((eSize - tokenFigure.getPreferredSize().width) / 2, 
								(eSize - tokenFigure.getPreferredSize().height) / 2, 
								-1, -1));
//		} else {
//			// Label ist breiter als Ellipse!
//			// Der Place steht mittig ber dem Label
//
//			// Der Place steht mittig ber dem Label
//			getLayoutManager().setConstraint(e, new Rectangle(
//					labelEllipseDiff / 2, 0, eSize, eSize));
//			// Label fllt die ganze Figur am unteren Rand aus
//			getLayoutManager().setConstraint(label,
//					new Rectangle(0, eSize + 3, -1, -1));
//
//			getLayoutManager().setConstraint(tokenFigure, new Rectangle(
//					(label.getPreferredSize().width - tokenFigure.getPreferredSize().width) / 2, 
//					(eSize - tokenFigure.getPreferredSize().height) / 2,
//					-1, -1));
//
//		}
		super.validate();
	}
	
	
	public EllipseAnchor getSourceCBConnectionAnchor(){
		return outgoingCBConnectionAnchor;
	}
	
	public EllipseAnchor getTargetCBConnectionAnchor(){
		return incomingCBConnectionAnchor;
	}
	
	
	/* (non-Javadoc)
	 * @see org.eclipse.draw2d.Figure#useLocalCoordinates()
	 */
	@Override
	protected boolean useLocalCoordinates() {
		return true;
	}

	
	/* (non-Javadoc)
	 * @see org.eclipse.draw2d.IFigure#getMinimumSize(int, int)
	 */
	@Override
	public Dimension getMinimumSize(int wHint, int hHint) {	
		return getPreferredSize();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.draw2d.IFigure#getPreferredSize(int, int)
	 */
	@Override
	public Dimension getPreferredSize(int wHint, int hHint) {
		if (mappingLabel.getSize().width > 0) {
			return new Dimension(eSize + mappingLabel.getSize().width+3, eSize);
		} else {
			return new Dimension(eSize, eSize);
		}
	}

	/**
	 * This method creates the PAnchorPoints at the Node
	 */
	protected void createAnchors() {
		e.createAnchors();	
	}

	/**
	 * This method refreshes the Position of the AnchorPoints at the Node as
	 * soon as there has been occured a changing of the dimension
	 */
	protected void refreshAnchors() {
		e.refreshAnchors();
	}

	/**
	 * Returns a list of anchors for source connections
	 * 
	 * @return
	 */
	public List<ONAnchor> getSourceConnectionAnchors() {
		return outgoingConnectionAnchors;
	}
	
	/**
	 * Returns a list of anchors for target connections.
	 * 
	 * @return
	 */
	public List<ONAnchor> getTargetConnectionAnchors() {
		return incomingConnectionAnchors;
	}
	
	/**
	 * Detection of the closest source connection anchor around the mouse
	 * pointer
	 * 
	 * @return the anchor which is closest to the actual mouse position
	 */
	public ConnectionAnchor getSourceConnectionAnchorAt(Point mousePoint) {
		return getClosestConnectionAnchorAt(mousePoint,
				getSourceConnectionAnchors(),getSourceCBConnectionAnchor());
	}

	/**
	 * Detection of the closest target connection anchor around the mouse
	 * pointer
	 * 
	 * @return the anchor which is closest to the actual mouse position
	 */
	public ConnectionAnchor getTargetConnectionAnchorAt(Point mousePoint) {
		return getClosestConnectionAnchorAt(mousePoint,
				getTargetConnectionAnchors(),getTargetCBConnectionAnchor());
	}

	/**
	 * Calculating the minimal distant anchor regarding to the mouse pointer
	 * 
	 * @param p
	 * @param connectionAnchors
	 * @return the closest anchor around the point p
	 */
	static private ConnectionAnchor getClosestConnectionAnchorAt(Point p,
			List<ONAnchor> connectionAnchors, EllipseAnchor cba) {

		// initializing variables
		ConnectionAnchor closest = null;
		long min = Long.MAX_VALUE;
		int anz_Anchors = connectionAnchors.size();
		Point p2 = new Point(0, 0);

		// checking each anchor point and taking the best one
		for (int i = 0; i < anz_Anchors; i++) {
			ONAnchor a = connectionAnchors.get(i);

			// absolut coordinates of the anchor
			p2 = a.getLocation(null);

			long d = p.getDistance2(p2);
			if (d < min) {
				min = d;
				closest = a;
			}
		}
		//chopbox anchor
		p2 = cba.getReferencePoint();
		long d = p.getDistance2(p2);
		if (d < min) {
			min = d;
			closest = cba;
		}
		return closest;
	}
	
	protected class EllipseFigure extends Ellipse{		
				
		public EllipseFigure(){
			super();
			StackLayout l = new StackLayout();
			this.setLayoutManager(l);
			this.setFill(true);
		}
		
		/**
		 * This method creates the PAnchorPoints at the Node
		 */
		protected void createAnchors() {

			// 10 anchors around the transition figure
			//Dimension dF = new Dimension(eSize,eSize); //getPreferredSize(-1, -1);
			Point[] connectionPoints = new Point[12];
			
			
			Point relativerMittelPunkt = new Point(eSize/2,eSize/2);
			
			// rechts
			connectionPoints[0] = new Point(eSize/2*Math.cos(0),eSize/2*Math.sin(0));
			// oben
			connectionPoints[1] = new Point(eSize/2*Math.cos(Math.PI/2),eSize/2*Math.sin(Math.PI/2));
			// links
			connectionPoints[2] = new Point(eSize/2*Math.cos(Math.PI),eSize/2*Math.sin(Math.PI));
			// rechts
			connectionPoints[3] = new Point(eSize/2*Math.cos(Math.PI*3/2),eSize/2*Math.sin(Math.PI*3/2));
			
			
			// 2 weitere zwischen rechts und oben
			connectionPoints[4] = new Point(eSize/2*Math.cos(Math.PI/2*1/3),eSize/2*Math.sin(Math.PI/2*1/3));
			connectionPoints[5] = new Point(eSize/2*Math.cos(Math.PI/2*2/3),eSize/2*Math.sin(Math.PI/2*2/3));
			// 2 weitere zwischen oben und links
			connectionPoints[6] = new Point(eSize/2*Math.cos(Math.PI/2+Math.PI/2*1/3),eSize/2*Math.sin(Math.PI/2+Math.PI/2*1/3));
			connectionPoints[7] = new Point(eSize/2*Math.cos(Math.PI/2+Math.PI/2*2/3),eSize/2*Math.sin(Math.PI/2+Math.PI/2*2/3));
			// 2 weitere zwischen links und unten
			connectionPoints[8] = new Point(eSize/2*Math.cos(Math.PI+Math.PI/2*1/3),eSize/2*Math.sin(Math.PI+Math.PI/2*1/3));
			connectionPoints[9] = new Point(eSize/2*Math.cos(Math.PI+Math.PI/2*2/3),eSize/2*Math.sin(Math.PI+Math.PI/2*2/3));
			// 2 weitere zwischen unten und rechts
			connectionPoints[10] = new Point(eSize/2*Math.cos(Math.PI*3/2+Math.PI/2*1/3),eSize/2*Math.sin(Math.PI*3/2+Math.PI/2*1/3));
			connectionPoints[11] = new Point(eSize/2*Math.cos(Math.PI*3/2+Math.PI/2*2/3),eSize/2*Math.sin(Math.PI*3/2+Math.PI/2*2/3));
			

			ONAnchor tempA;
			for (int i = 0; i < 12; i++) {
				tempA = new ONAnchor(e);
				tempA.setRelativeDist(relativerMittelPunkt.getTranslated(connectionPoints[i]));
				tempA.setId(i);
				incomingConnectionAnchors.add(tempA);
				outgoingConnectionAnchors.add(tempA);

			}
		}

		/**
		 * This method refreshes the Position of the AnchorPoints at the Node as
		 * soon as there has been occured a changing of the dimension
		 */
		protected void refreshAnchors() {

			//Dimension dF = getPreferredSize(-1, -1);
			Point[] connectionPoints = new Point[12];

			
			
			Point relativerMittelPunkt = new Point(eSize/2,eSize/2);
			
			// rechts
			connectionPoints[0] = new Point(eSize/2*Math.cos(0),eSize/2*Math.sin(0));
			// oben
			connectionPoints[1] = new Point(eSize/2*Math.cos(Math.PI/2),eSize/2*Math.sin(Math.PI/2));
			// links
			connectionPoints[2] = new Point(eSize/2*Math.cos(Math.PI),eSize/2*Math.sin(Math.PI));
			// rechts
			connectionPoints[3] = new Point(eSize/2*Math.cos(Math.PI*3/2),eSize/2*Math.sin(Math.PI*3/2));
			
			
			// 2 weitere zwischen rechts und oben
			connectionPoints[4] = new Point(eSize/2*Math.cos(Math.PI/2*1/3),eSize/2*Math.sin(Math.PI/2*1/3));
			connectionPoints[5] = new Point(eSize/2*Math.cos(Math.PI/2*2/3),eSize/2*Math.sin(Math.PI/2*2/3));
			// 2 weitere zwischen oben und links
			connectionPoints[6] = new Point(eSize/2*Math.cos(Math.PI/2+Math.PI/2*1/3),eSize/2*Math.sin(Math.PI/2+Math.PI/2*1/3));
			connectionPoints[7] = new Point(eSize/2*Math.cos(Math.PI/2+Math.PI/2*2/3),eSize/2*Math.sin(Math.PI/2+Math.PI/2*2/3));
			// 2 weitere zwischen links und unten
			connectionPoints[8] = new Point(eSize/2*Math.cos(Math.PI+Math.PI/2*1/3),eSize/2*Math.sin(Math.PI+Math.PI/2*1/3));
			connectionPoints[9] = new Point(eSize/2*Math.cos(Math.PI+Math.PI/2*2/3),eSize/2*Math.sin(Math.PI+Math.PI/2*2/3));
			// 2 weitere zwischen unten und rechts
			connectionPoints[10] = new Point(eSize/2*Math.cos(Math.PI*3/2+Math.PI/2*1/3),eSize/2*Math.sin(Math.PI*3/2+Math.PI/2*1/3));
			connectionPoints[11] = new Point(eSize/2*Math.cos(Math.PI*3/2+Math.PI/2*2/3),eSize/2*Math.sin(Math.PI*3/2+Math.PI/2*2/3));
			
			
			for (int i = 0; i < 12; i++) {
				incomingConnectionAnchors.get(i).setRelativeDist(relativerMittelPunkt.getTranslated(connectionPoints[i]));
				outgoingConnectionAnchors.get(i).setRelativeDist(relativerMittelPunkt.getTranslated(connectionPoints[i]));
			}
		}

		
	}

	
}
