package roneditor.ronview;

import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.EditPartFactory;
import org.eclipse.gef.EditPartViewer;
import org.eclipse.gef.KeyHandler;
import org.eclipse.gef.KeyStroke;
import org.eclipse.gef.palette.PaletteRoot;
import org.eclipse.gef.ui.actions.DeleteAction;
import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.swt.SWT;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.actions.ActionFactory;
import org.eclipse.ui.part.WorkbenchPart;

import roneditor.AbstractMultiViewerPage;
import roneditor.AbstractPageBookView;
import roneditor.actions.AddTokenAction;
import roneditor.actions.FireTransitionAction;
import roneditor.editor.ContextMenuProviderWithActionRegistry;
import roneditor.editor.RONEditor;
import roneditor.objectnetview.actions.MoveNodeAction;
import roneditor.ronview.actions.CompleteMappingConsistencyCheckAction;
import roneditor.ronview.actions.DeleteHLBendPointAction;
import roneditor.ronview.edit.utils.RONViewEditPartFactory;
import ronmodel.HLNode;
import ronmodel.RON;

/**
 * <p>
 * This is an implementation of {@link AbstractPageBookView} to display
 * {@link RON}s.
 * </p>
 * 
 * <p>
 * It uses {@link RONViewEditPartFactory} and
 * {@link RONPageContextMenuProvider}, configures the keyhandler and
 * creates actions to move {@link HLNode}s by pressing keys.
 * </p>
 * 
 * <p>
 * On model change nothing is done.
 * </p>
 * 
 * @author Tony Modica
 */
public class RONPage extends AbstractMultiViewerPage {

	public RONPage(final RON ron, final IEditorPart editor) {
		super(ron, editor);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see roneditor.objectnetview.AbstractPageWithFlyoutPalette#createActions()
	 */
	@Override
	protected void createActions() {
	
		// HINT Hier werden Instanzen der Actions erzeugt und registriert,
		// damit der ContextMenuProvider sie in das Kontextmen einfgen kann.

		SelectionAction selectionAction;
		selectionAction = new MoveNodeAction(getEditor(), MoveNodeAction.LEFT);
		getActionRegistry().registerAction(selectionAction);

		selectionAction = new MoveNodeAction(getEditor(), MoveNodeAction.RIGHT);
		getActionRegistry().registerAction(selectionAction);

		selectionAction = new MoveNodeAction(getEditor(), MoveNodeAction.UP);
		getActionRegistry().registerAction(selectionAction);

		selectionAction = new MoveNodeAction(getEditor(), MoveNodeAction.DOWN);
		getActionRegistry().registerAction(selectionAction);

		selectionAction = new MoveNodeAction(getEditor(),
				MoveNodeAction.PREC_LEFT);
		getActionRegistry().registerAction(selectionAction);

		selectionAction = new MoveNodeAction(getEditor(),
				MoveNodeAction.PREC_RIGHT);
		getActionRegistry().registerAction(selectionAction);

		selectionAction = new MoveNodeAction(getEditor(),
				MoveNodeAction.PREC_UP);
		getActionRegistry().registerAction(selectionAction);

		selectionAction = new MoveNodeAction(getEditor(),
				MoveNodeAction.PREC_DOWN);
		getActionRegistry().registerAction(selectionAction);
		
		// Delete Action
		getActionRegistry().registerAction(new DeleteAction((WorkbenchPart)getEditor()));
		registerHandleForEditorAction(ActionFactory.DELETE.getId());
		
		getActionRegistry().registerAction(new CompleteMappingConsistencyCheckAction(getEditor()));
		registerHandleForEditorAction(CompleteMappingConsistencyCheckAction.ID);
		
		getActionRegistry().registerAction(new AddTokenAction(getEditor()));
		registerHandleForEditorAction(AddTokenAction.ID);
		
		getActionRegistry().registerAction(new FireTransitionAction((RONEditor)getEditor()));
		registerHandleForEditorAction(FireTransitionAction.ID);
		
		getActionRegistry().registerAction(new DeleteHLBendPointAction(getEditor()));
		registerHandleForEditorAction(DeleteHLBendPointAction.ID);
		
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see roneditor.objectnetview.AbstractPageWithFlyoutPalette#setupKeyHandler(org.eclipse.gef.KeyHandler)
	 */
	@Override
	protected void setupKeyHandler(final KeyHandler kh) {
		// Arrow keys run the correspondent MoveNodeActions
		kh.put(KeyStroke.getPressed(SWT.ARROW_LEFT, 0), (getActionRegistry()
				.getAction(MoveNodeAction.LEFT)));
		kh.put(KeyStroke.getPressed(SWT.ARROW_RIGHT, 0), (getActionRegistry()
				.getAction(MoveNodeAction.RIGHT)));
		kh.put(KeyStroke.getPressed(SWT.ARROW_UP, 0), (getActionRegistry()
				.getAction(MoveNodeAction.UP)));
		kh.put(KeyStroke.getPressed(SWT.ARROW_DOWN, 0), (getActionRegistry()
				.getAction(MoveNodeAction.DOWN)));

		// SHIFT + Arrow keys run the correspondent precise MoveNodeActions
		kh.put(KeyStroke.getPressed(SWT.ARROW_LEFT, SWT.SHIFT),
				(getActionRegistry().getAction(MoveNodeAction.PREC_LEFT)));
		kh.put(KeyStroke.getPressed(SWT.ARROW_RIGHT, SWT.SHIFT),
				(getActionRegistry().getAction(MoveNodeAction.PREC_RIGHT)));
		kh.put(KeyStroke.getPressed(SWT.ARROW_UP, SWT.SHIFT),
				(getActionRegistry().getAction(MoveNodeAction.PREC_UP)));
		kh.put(KeyStroke.getPressed(SWT.ARROW_DOWN, SWT.SHIFT),
				(getActionRegistry().getAction(MoveNodeAction.PREC_DOWN)));
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see roneditor.objectnetview.AbstractPageWithFlyoutPalette#createEditPartFactory()
	 */
	@Override
	protected EditPartFactory createEditPartFactory() {
		return new RONViewEditPartFactory();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see roneditor.objectnetview.AbstractPageWithFlyoutPalette#getContextMenuProvider()
	 */
	@Override
	protected ContextMenuProviderWithActionRegistry createContextMenuProvider(
			final EditPartViewer viewer) {
		return new RONPageContextMenuProvider(viewer, getActionRegistry());
	}
	
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see roneditor.objectnetview.AbstractPageWithFlyoutPalette#createPaletteRoot()
	 */
	@Override
	protected PaletteRoot createPaletteRoot() {
		// just an empty palette root
		return new RONPaletteRoot();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see roneditor.objectnetview.AbstractPageWithFlyoutPalette#getThumbSashWeights()
	 */
	@Override
	protected int[] getThumbSashWeights() {
		return new int[] { 3, 1 };
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.emf.common.notify.Adapter#notifyChanged(org.eclipse.emf.common.notify.Notification)
	 */
	public void notifyChanged(final Notification notification) {
		// the page doesn't care for model changes (yet)
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see roneditor.ruleview.AbstractMultiViewerPage#getNumberOfViewers()
	 */
	@Override
	public int getNumberOfViewers() {
		return 1;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see roneditor.ruleview.AbstractMultiViewerPage#getViewerContents()
	 */
	@Override
	public EObject[] getViewerContents() {
		return new EObject[] { getModel() };
	}

}