package roneditor.ronview;

import org.eclipse.gef.EditPartViewer;
import org.eclipse.gef.ui.actions.ActionRegistry;
import org.eclipse.gef.ui.actions.GEFActionConstants;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.actions.ActionFactory;

import roneditor.actions.AddTokenAction;
import roneditor.actions.FireTransitionAction;
import roneditor.editor.ContextMenuProviderWithActionRegistry;
import roneditor.objectnetview.actions.EditAction;
import roneditor.ronview.actions.DeleteHLBendPointAction;

/** 
 * Context menu provider class.
 */
public class RONPageContextMenuProvider extends ContextMenuProviderWithActionRegistry {
	/** the action registry */
	private final ActionRegistry actionRegistry;

	/**
	 * Creates a new WorkflowEditorContextMenuProvider instance.
	 * @param viewer
	 */
	public RONPageContextMenuProvider(
		EditPartViewer viewer,
		ActionRegistry actionRegistry) {
		super(viewer, actionRegistry);
		this.actionRegistry = actionRegistry;
		
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.ContextMenuProvider#buildContextMenu(org.eclipse.jface.action.IMenuManager)
	 */
	public void buildContextMenu(IMenuManager menuManager) {
		GEFActionConstants.addStandardActionGroups(menuManager);

		appendActionToUndoGroup(menuManager, ActionFactory.UNDO.getId());
		appendActionToUndoGroup(menuManager, ActionFactory.REDO.getId());

//		appendActionToEditGroup(menuManager, ActionFactory.COPY.getId());
//		appendActionToEditGroup(menuManager, ActionFactory.PASTE.getId());
		appendActionToEditGroup(menuManager, ActionFactory.DELETE.getId());
		appendActionToEditGroup(menuManager, GEFActionConstants.DIRECT_EDIT);
		appendActionToAddGroup(menuManager,EditAction.ID);
		appendActionToAddGroup(menuManager, AddTokenAction.ID);
		appendActionToAddGroup(menuManager, FireTransitionAction.ID);
		appendActionToAddGroup(menuManager, DeleteHLBendPointAction.ID);
		
		appendActionToMenu(
			menuManager,
			ActionFactory.SAVE.getId(),
			GEFActionConstants.GROUP_SAVE);
	}


	/**
	 * Returns the action registry.
	 * @return the action registry
	 */
	protected ActionRegistry getActionRegistry() {
		return actionRegistry;
	}

	/**
	 * Appends the specified action to the specified menu group
	 * @param actionId
	 * @param menuGroup
	 */
	private void appendActionToMenu(
		IMenuManager menu,
		String actionId,
		String menuGroup) {
		IAction action = getActionRegistry().getAction(actionId);
		if (null != action && action.isEnabled())
			menu.appendToGroup(menuGroup, action);
	}

	/**
	 * Appends the specified action to the specified menu group
	 * @param actionId
	 * @param menuGroup
	 */
	private void appendActionToUndoGroup(IMenuManager menu, String actionId) {
		IAction action = getActionRegistry().getAction(actionId);
		if (null != action && action.isEnabled())
			menu.appendToGroup(GEFActionConstants.GROUP_UNDO, action);
	}

	/**
	 * Appends the specified action to the specified menu group
	 * @param actionId
	 * @param menuGroup
	 */
	private void appendActionToEditGroup(IMenuManager menu, String actionId) {
		IAction action = getActionRegistry().getAction(actionId);
		if (null != action && action.isEnabled())
			menu.appendToGroup(GEFActionConstants.GROUP_EDIT, action);
	}

	/**
	 * Appends the specified action to the specified menu group
	 * @param actionId
	 * @param menuGroup
	 */
	private void appendActionToAddGroup(IMenuManager menu, String actionId) {
		IAction action = getActionRegistry().getAction(actionId);
		if (null != action /*&& action.isEnabled()*/)
			menu.appendToGroup(IWorkbenchActionConstants.GROUP_ADD, action);
	}

}
