package roneditor.ronview.anchors;

import org.eclipse.draw2d.AbstractConnectionAnchor;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.PrecisionPoint;
import org.eclipse.draw2d.geometry.Rectangle;

	public class HLAnchor
		extends AbstractConnectionAnchor
	{
		
		private Point relativeDist;
		private int id;

	/**
	 * Constructs a HLAnchor with the given <i>owner</i> figure.
	 * 
	 * @param owner The owner figure
	 * @since 2.0
	 */
	public HLAnchor(IFigure owner) {
		super(owner);
	}

	public Point getLocation(Point reference) {
		Rectangle r = getOwner().getBounds();

		int x, y;
		x = r.x + relativeDist.x;
		y = r.y + relativeDist.y;

		Point p = new PrecisionPoint(x, y);
		getOwner().translateToAbsolute(p);
		return p;
	}

	/**
	 * Returns the bounds of this HLAnchor's owner.  Subclasses can override this method
	 * to adjust the box the anchor can be placed on.  For instance, the owner figure may have
	 * a drop shadow that should not be included in the box. 
	 *  
	 * @return The bounds of this HLAnchor's owner
	 * @since 2.0
	 */
	protected Rectangle getBox() {
		return getOwner().getBounds();
	}

	/**
	 * Returns the anchor's reference point. In the case of the HLAnchor, this is the
	 * center of the anchor's owner.
	 * 
	 * @return The reference point
	 */
	public Point getReferencePoint() {
		return getLocation(null);
	}

	/**
	 * Returns the relative distance
	 * 
	 * @return
	 */
	public Point getRelativeDist() {
		return relativeDist;
	}

	/**
	 * Sets the relative distance
	 * 
	 * @param relativeDist
	 */
	public void setRelativeDist(Point relativeDist) {
		this.relativeDist = relativeDist;
	}
	
	/**
	 * Returns the id
	 * 
	 * @return
	 */
	public int getId() {
		return id;
	}

	/**
	 * Sets the id
	 * 
	 * @param i
	 */
	public void setId(int i) {
		id = i;
	}

}
