package roneditor.ronview.commands;

import org.eclipse.gef.commands.Command;

import ronmodel.HLArc;
import ronmodel.HLNode;
import ronmodel.HLTransition;
import ronmodel.RON;

public abstract class AbstractHLConnectionCommand extends Command {

	private static final String ConnectionCommand_Label = "HLArc";
	
	protected HLArc hlArc;
	protected HLNode source;
	protected HLNode target;
	//protected Integer weight;
	
	protected int anchorNrT;
	protected int anchorNrS;
	
		
	/**
	 * Constructor.
	 */
	public AbstractHLConnectionCommand() {
		super(ConnectionCommand_Label);
	}

	public abstract boolean canExecute();
	
	/**
	 * Entry point for the command execution.
	 */
	public void execute() {
		//hlArc.setWeight(weight);
		hlArc.setAnchorNrS(anchorNrS);
		hlArc.setAnchorNrT(anchorNrT); 
		RON on = target.getRon();
		hlArc.setRon(on);
		if (source instanceof HLTransition){			
			hlArc.setTarget(target);
			hlArc.setSource(source);
		} else {
			hlArc.setSource(source);
			hlArc.setTarget(target);
		}
	}
	
	/**
	 * Undo command execution.
	 */
	public void undo() {
		hlArc.setRon(null);
		hlArc.setSource(null);
		hlArc.setTarget(null);
	}
	
	/**
	 * Redo command execution.
	 */
	public void redo() {
		execute();
	}

	
	/**
	 * @return Returns the anchor number of the source.
	 */
	public int getAnchorNrS() {
		return anchorNrS;
	}// getAnchorNrS

	/**
	 * @return Returns the anchor number of the target.
	 */
	public int getAnchorNrT() {
		return anchorNrT;
	}// getAnchorNrT

	/**
	 * @param anchorNrS
	 *            The anchor number of the source to set.
	 */
	public void setAnchorNrS(int anchorNrS) {
		this.anchorNrS = anchorNrS;
	}// setAnchorNrS

	/**
	 * @param anchorNrT
	 *            The anchor number of the source to set.
	 */
	public void setAnchorNrT(int anchorNrT) {
		this.anchorNrT = anchorNrT;
	}// setAnchorNrT

	public void setHLArc(HLArc onarc) {
		this.hlArc = onarc;
	}

	/**
	 * @return the onarc
	 */
	public HLArc getHLArc() {
		return hlArc;
	}

	protected void setSource(HLNode source) {
		this.source = source;
	}

	protected void setTarget(HLNode target) {
		this.target = target;
	}

	/**
	 * @return the source
	 */
	public HLNode getSource() {
		return source;
	}

	/**
	 * @return the target
	 */
	public HLNode getTarget() {
		return target;
	}

//	/**
//	 * @return the weight
//	 */
//	public Integer getWeight() {
//		return this.weight;
//	}
//
//	/**
//	 * @param weight the weight to set
//	 */
//	public void setWeight(Integer weight) {
//		this.weight = weight;
//	}

}
