/**
 * 
 */
package roneditor.ronview.commands;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.StringTokenizer;

import org.eclipse.gef.commands.Command;

import ronmodel.HLPlace;
import ronmodel.ONArc;
import ronmodel.ONNode;
import ronmodel.ObjectNet;

/**
 * @author Mario
 *
 */
public class MergeObjectNetsCommand extends Command {
	List<ObjectNet> objNets;
	HashMap<ONNode,Integer> onNodeMap;
	HashMap<ONArc,Integer> onArcMap;
	HashMap<Integer,HLPlace> hlPlacesMap;
	
	public MergeObjectNetsCommand() {
		super();
		objNets = new ArrayList<ObjectNet>();
		onNodeMap = new HashMap<ONNode,Integer>();
		onArcMap = new HashMap<ONArc,Integer>();
		hlPlacesMap = new HashMap<Integer,HLPlace>();
	}


	/**
	 * Executes the command by adding all Nodes / Arcs to first ObjectNet.
	 */
	@Override
	public void execute() {			
		for (int i = 1; i < objNets.size(); i++) {
			// fr jeden ONNode sich merken in welches ObjectNet er gehrt
			for (Object obj:objNets.get(i).getNodes()) {
				onNodeMap.put((ONNode)obj,i);
			}
			// fr jeden ONArc sich merken in welches ObjectNet er gehrt
			for (Object obj:objNets.get(i).getArcs()) {
				onArcMap.put((ONArc)obj,i);
			}
			objNets.get(0).getNodes().addAll(objNets.get(i).getNodes());
			objNets.get(0).getArcs().addAll(objNets.get(i).getArcs());
			hlPlacesMap.put(i,objNets.get(i).getPlace());
			objNets.get(i).setPlace(null);
			// Name setzen
			objNets.get(0).setName(objNets.get(0).getName()+"|"+objNets.get(i).getName());
		}
	}
	
	
	
	/**
	 * Undoes the command by using the HashMap which saves the corresponding ObjectNetNumber.
	 */
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.commands.Command#undo()
	 */
	@Override
	public void undo() {
		// Knoten
		for (int i = 0; i < objNets.get(0).getNodes().size();) {
			ONNode onNode = (ONNode)objNets.get(0).getNodes().get(i);
			Integer objNetNumber = onNodeMap.get(onNode);
			if (objNetNumber != null) { 
			// wenns null ist, dann ist es ein Knoten aus dem ersten ObjectNetz und muss nicht umgehangen werden
				objNets.get(objNetNumber).getNodes().add(onNode);
			} else {
				i++;
			}
		}
		
		// Kanten
		for (int i = 0; i < objNets.get(0).getArcs().size();) {
			ONArc onArc = (ONArc)objNets.get(0).getArcs().get(i);;
			Integer objNetNumber = onArcMap.get(onArc);
			if (objNetNumber != null) { 
			// wenns null ist, dann ist es ein Knoten aus dem ersten ObjectNetz und muss nicht umgehangen werden
				objNets.get(objNetNumber).getArcs().add(onArc);
			} else {
				i++;
			}
		}
		
		for (int i=1; i < objNets.size(); i++) {
			objNets.get(i).setPlace(hlPlacesMap.get(i));
		}

		StringTokenizer strTokenizer = new StringTokenizer(objNets.get(0).getName(),"|");
		for (int i=0; i < objNets.size(); i++) {
		// Name setzen
			if (strTokenizer.hasMoreTokens()) {
				objNets.get(i).setName(strTokenizer.nextToken());
			} else {
				objNets.get(i).setName(objNets.get(0).getName());
			}
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.commands.Command#redo()
	 */
	@Override
	public void redo() {

		onNodeMap.clear();
		onArcMap.clear();
		hlPlacesMap.clear();
		execute();
	}


	/**
	 * @param objNets the objNets to set
	 */
	public void setObjNets(List<ObjectNet> objNets) {
		this.objNets = objNets;
	}
	

}
