package roneditor.ronview.commands;

import org.eclipse.gef.commands.Command;

import ronmodel.HLArc;
import ronmodel.HLNode;

public class ReconnectHLArcCommand extends Command{

		private HLArc onArc;

		private HLNode newSource;

		private HLNode newTarget;

		private final HLNode oldSource;

		private final HLNode oldTarget;

		private int newAnchorNrT;

		private int newAnchorNrS;

		private int oldAnchorNrT;

		private int oldAnchorNrS;

		/**
		 * Constructor
		 * 
		 * @param onArc
		 */
		public ReconnectHLArcCommand(HLArc onArc) {
			this.onArc = onArc;
			this.oldSource = onArc.getSource();
			this.oldTarget = onArc.getTarget();
			this.oldAnchorNrS = onArc.getAnchorNrS();
			this.oldAnchorNrT = onArc.getAnchorNrT();
		}// constructor

		/**
		 * Executes the command by setting the new source or target for the given
		 * HLArc, depending on what has been specified.
		 * 
		 * @see org.eclipse.gef.commands.Command#execute()
		 */
		public void execute() {
			// is the source to be set or the target?
			if (newSource != null) {
				// setting new source
				onArc.setSource(newSource);				
				onArc.setAnchorNrS(newAnchorNrS);
			} else if (newTarget != null) {
				// setting new target
				onArc.setTarget(newTarget);				
				onArc.setAnchorNrT(newAnchorNrT);
			}//if else
		}// execute


		/**
		 * Undoes the command by resetting the old source or target for the given
		 * onArc which has been saved by {@link #execute()}, depending on if a new
		 * source or target has been specified
		 * 
		 * @see org.eclipse.gef.commands.Command#undo()
		 */
		public void undo() {
			if (newSource != null) {
				onArc.setSource(oldSource);
				onArc.setAnchorNrS(oldAnchorNrS);
			} else if (newTarget != null) {
				onArc.setTarget(oldTarget);
				onArc.setAnchorNrT(oldAnchorNrT);
			}//if else
		}// undo


		/**
		 * @param newSource
		 *            The newSource to set. newTarget is set to null.
		 */
		public void setNewSource(HLNode newSource) {
			this.newSource = newSource;
			newTarget = null;
		}// setNewSource


		/**
		 * @param newTarget
		 *            The newTarget to set. newSource is set to null.
		 */
		public void setNewTarget(HLNode newTarget) {
			this.newTarget = newTarget;
			newSource = null;
		}// setNewTarget


		/**
		 * @param AnchorNrT
		 *            The number of the new target anchor
		 */
		public void setNewAnchorNrT(int AnchorNrT) {
			this.newAnchorNrT = AnchorNrT;
		}// setNumberOfAnchorT


		/**
		 * @param AnchorNrS
		 *            The number of the new source anchor
		 */
		public void setNewAnchorNrS(int AnchorNrS) {
			this.newAnchorNrS = AnchorNrS;
		}// setNumberOfAnchorS
		
		public HLNode getNewSource(){
			return newSource;
		}
		
		public HLNode getNewTarget(){
			return newTarget;
		}
		
		public HLArc getHLArc(){
			return onArc;
		}

		public HLNode getOldSource(){
			return oldSource;
		}
		
		public HLNode getOldTarget(){
			return oldTarget;
		}
		
		public int getNewAnchorNrS(){
			return newAnchorNrS;
		}
		
		public int getNewAnchorNrT(){
			return newAnchorNrT;
		}
		
		public int getOldAnchorNrS(){
			return oldAnchorNrS;
		}
		
		public int getOldAnchorNrT(){
			return oldAnchorNrT;
		}
		

}
