/**
 * 
 */
package roneditor.ronview.commands;

import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

import org.eclipse.gef.commands.Command;

import roneditor.model.RONModelManager;
import ronmodel.HLPlace;
import ronmodel.ONArc;
import ronmodel.ONNode;
import ronmodel.ObjectNet;

/**
 * @author Mario
 *
 */
public class SplitObjectNetCommand extends Command {
	HLPlace oldHLPlace, newHLPlace;
	ObjectNet objNet;
	List<ObjectNet> objNets;
	String oldONName=null;
	
	public SplitObjectNetCommand() {
		super();
		objNets = new ArrayList<ObjectNet>();
	}


	/**
	 * Executes the command by splitting the objNet.
	 */
	@Override
	public void execute() {	
		
		List<ONNode> allONNodes = new ArrayList<ONNode>(objNet.getNodes());
		List<ArrayList<ONNode>> splitONNodes = new ArrayList<ArrayList<ONNode>>();
		List<ArrayList<ONArc>> splitONArcs = new ArrayList<ArrayList<ONArc>>();
		Integer counter = 0;


		StringTokenizer strTokenizer = new StringTokenizer(objNet.getName(),"|");
		
		
		// erst aufhren wenn alle ONNodes neu eingeordnet wurde
		while (!allONNodes.isEmpty()) {
			// den ersten Knoten rausnehmen
			ONNode onNode = allONNodes.remove(0);
			// neue Liste erzeugen und ihn dort als ersten einfgen
			splitONNodes.add(new ArrayList<ONNode>());
			// neue Liste fr die Kanten erstellen die zu diesem TeilObjectNetz gehren
			splitONArcs.add(new ArrayList<ONArc>());
			splitONNodes.get(counter).add(onNode);
			Boolean done = false;
			Integer runner = 0;
			// solange man einen neuen ONNode findet der erreichbar ist, weitermachen
			while (!done) {
				List<ONNode> list = splitONNodes.get(counter); 
				onNode = list.get(runner);
				List<ONArc> incONArcs= onNode.getIncoming();
				List<ONArc> outONArcs= onNode.getOutgoing();
				
									
				for (ONArc onArc: incONArcs) {
					ONNode newONNode = onArc.getSource();
					if (!splitONNodes.get(counter).contains(newONNode)) {
						allONNodes.remove(newONNode);
						splitONNodes.get(counter).add(newONNode);
					}
					if (!splitONArcs.get(counter).contains(onArc)) {
						splitONArcs.get(counter).add(onArc);
					}					
				}
				
				for (ONArc onArc: outONArcs) {
					ONNode newONNode = onArc.getTarget();
					if (!splitONNodes.get(counter).contains(newONNode)) {
						allONNodes.remove(newONNode);
						splitONNodes.get(counter).add(newONNode);
					}
					if (!splitONArcs.get(counter).contains(onArc)) {
						splitONArcs.get(counter).add(onArc);
					}
				}
				
				// dies ist genau dann der Fall, wenn man am Ende der Liste ist und bei Abarbeitung dieses Knotens keine neuen Knoten hinzukamen
				if (runner == splitONNodes.get(counter).size()-1) {
					done = true;
				}
				runner++;
				
			}
			counter++;			
		}
		
		
		// splitONNodes und splitONArcs sollten nu die getrennten Mengen halten
		// bei dem ersten ObjectNetz muss kein neues ObjectNetz erstellt werden
		if (!splitONNodes.isEmpty()) {
			objNet.getNodes().clear();
			objNet.getNodes().addAll(splitONNodes.get(0));
			objNet.getArcs().clear();
			objNet.getArcs().addAll(splitONArcs.get(0));
			objNets.add(objNet);
		}
		
		HLPlace parentHLPlace = objNet.getPlace();
		
		// Name frs ersten ObjectNetz setzen
		objNet.setName(strTokenizer.nextToken());
		
		// fr alle anderen muss ein neues ObjectNetz erstellt werden
		for (int i=1; i < splitONNodes.size(); i++) {
			ObjectNet newObjNet= RONModelManager.getFactory().createObjectNet();
			newObjNet.getNodes().addAll(splitONNodes.get(i));
			newObjNet.getArcs().addAll(splitONArcs.get(i));
			if (strTokenizer.hasMoreTokens()) {
				newObjNet.setName(strTokenizer.nextToken());
			} else {
				newObjNet.setName(objNet.getName());
			}
			newObjNet.setPlace(parentHLPlace);
			objNets.add(newObjNet);			
		}
		
		
		
	}
	
	
	
	/**
	 * Undoes the command by removing the Token from the newHLPlace and adding it to the OldHLPlace.
	 */
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.commands.Command#undo()
	 */
	@Override
	public void undo() {
		for (int i = 1; i < objNets.size(); i++) {
			objNet.getNodes().addAll(objNets.get(i).getNodes());
			objNet.getArcs().addAll(objNets.get(i).getArcs());
			objNets.get(i).setPlace(null); // Lschen
			//objNet.setName(objNet.getName()+"|"+objNets.get(i).getName());
			objNet.setName(oldONName);
		}		
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.commands.Command#redo()
	 */
	@Override
	public void redo() {
		objNets.clear();
		execute();
	}
	


	
	/**
	 * @param newHLPlace the newHLPlace to set
	 */
	public void setNewHLPlace(HLPlace newHLPlace) {
		this.newHLPlace = newHLPlace;
	}


	/**
	 * @param objNet the objNet to set
	 */
	public void setObjNet(ObjectNet objNet) {
		this.objNet = objNet;
		this.oldONName = objNet.getName();
	}
	
}
