package roneditor.ronview.edit.parts;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.ConnectionAnchor;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.gef.ConnectionEditPart;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.NodeEditPart;
import org.eclipse.gef.Request;
import org.eclipse.gef.RequestConstants;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.requests.CreateConnectionRequest;
import org.eclipse.gef.requests.ReconnectRequest;
import org.eclipse.ui.views.properties.IPropertySource;

import roneditor.AbstractMultiViewerPage.MultiViewerPageViewer;
import roneditor.actions.AddTokenAction;
import roneditor.commands.CreateHLTokenCommand;
import roneditor.model.RONModelManager;
import roneditor.objectnetview.model.EObjectPropertySource;
import roneditor.ronview.edit.policies.HLNodeComponentEditPolicy;
import roneditor.ronview.edit.policies.HLPlaceEditPolicy;
import roneditor.ronview.edit.policies.HLPlaceXYLayoutEditPolicy;
import roneditor.ronview.figures.HLPlaceFigure;
import ronmodel.HLArc;
import ronmodel.HLPlace;
import ronmodel.HLPlaceType;
import ronmodel.HLTransition;
import ronmodel.LHS;
import ronmodel.NAC;
import ronmodel.ObjectNet;
import ronmodel.RHS;
import ronmodel.RonmodelPackage;
import ronmodel.Rule;
import _static_help_methods.RuleIDManager;

/**
 * EditPart for a HLPlace.
 */
public class HLPlaceEditPart 
	extends HLNodeEditPart
	implements NodeEditPart, Adapter
{
	private IPropertySource propertySource = null;
	private Notifier target;
	
	/**
	 * Creates a new HLPlaceEditPart instance.
	 * @param element
	 */
	public HLPlaceEditPart(HLPlace onPlace)
	{
		setModel(onPlace);
	}
	

	

	public HLPlace getHLPlace() {
		return (HLPlace)getModel();		
	}
	
	


	
	/* (non-Javadoc)
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#getModelSourceConnections()
	 */
	protected List getModelSourceConnections() {
		return getHLPlace().getOutgoing();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#getModelTargetConnections()
	 */
	protected List getModelTargetConnections() {
		return getHLPlace().getIncoming();
	}
	
	@Override
	protected List getModelChildren() {
		return getHLPlace().getTokens();
	}
	
	
	
	/**
	 * @author conny 
	 * 
	 * this methode add the figure of a Token to the figure of this HLPlace   
	 * 
	 * @param EditPart childEditPart - the EditPart of the current Token
	 * @param int index - index of the current Token in the list of all Tokens
	 */
	@Override
	protected void addChildVisual(EditPart childEditPart, int index){
		
		int maxChildrenNumber = maxNumberOfChildren(getAllPlacesOfRon());
		
		((HLPlaceFigure)this.getFigure()).addToE(
				((HLTokenEditPart)childEditPart).getFigure(), 
				index, 
				maxChildrenNumber);
	}
	
	
	/**
	 * @author conny
	 * 
	 * this methode refresh the Ellipse of the HLPlace-figure and its Token
	 */
	@Override
	protected void refreshChildren() {

		((HLPlaceFigure)this.getFigure()).removeAllChildrenFromE();
		List<HLTokenEditPart> token = this.getChildren();
		
		for(int i=0; i<token.size();i++){
			addChildVisual(token.get(i), i);
		}
		
		super.refreshChildren();
	}
	
	/**
	 * @author conny
	 * 
	 * this methode remove the figure of a Token
	 * 
	 * @param EditPart childEditPart - EditPart of the Token to remove
	 */
	@Override
	protected void removeChildVisual(EditPart childEditPart) {
		((HLPlaceFigure)this.getFigure()).removeChildFromE(((HLTokenEditPart)childEditPart).getFigure());
	}
	
	/**
	 * @author conny
	 * 
	 * this methode returns a list of all HLPlaces of the RON 
	 */
	private List<HLPlace> getAllPlacesOfRon(){
		List nodes = this.getHLPlace().getRon().getNodes();
		List<HLPlace> places = new ArrayList<HLPlace>();
		for(int i=0; i<nodes.size();i++){
			if(nodes.get(i) instanceof HLPlace){
				places.add((HLPlace)nodes.get(i));
			}
		}
		return places;
	}
	
	/**
	 * @author conny
	 * 
	 *  this methode returns a list of all HLPlaceEditParts of the RONEditPart
	 * 
	 */
	public List<HLPlaceEditPart> getAllPlaceEditParts(){
		List nodeEditParts = this.getParent().getChildren();
		List<HLPlaceEditPart> placeEditParts = new ArrayList<HLPlaceEditPart>();
		for(int i=0; i<nodeEditParts.size();i++){
			if(nodeEditParts.get(i) instanceof HLPlaceEditPart){
				placeEditParts.add((HLPlaceEditPart)nodeEditParts.get(i));
			}
		}
		return placeEditParts;
	}
	
	/**
	 * @author conny
	 * 
	 * this methode returns the maximum token number of a list of HLPlaces
	 */
	private int maxNumberOfChildren(List<HLPlace> places){
		int max=0;
		for(int i=0; i<places.size(); i++){
			max = Math.max(places.get(i).getTokens().size(), max);
		}
		return max;
	}
	
	/** 
	 * Returns the Figure of this, as a HLPlace type figure.
	 *
	 * @return  Figure as a HLPlaceFigure.
	 */
	protected HLPlaceFigure getHLPlaceFigure(){
		return (HLPlaceFigure) getFigure();
	}
	
	public void notifyChanged(Notification notification) {		
		int featureId = notification.getFeatureID( RonmodelPackage.class );
		switch( featureId ) {
			case RonmodelPackage.HL_PLACE__OUTGOING:				
				refreshSourceConnections();
				//refreshVisuals of all post-transitions (checkEnabled)
				for (Object o: getHLPlace().getOutgoing()){
					try{
						HLTransition t=(HLTransition)((HLArc)o).getTarget();
						for (Object o1: t.eAdapters()){
							if (o1 instanceof HLTransitionEditPart)
								((HLTransitionEditPart)o1).refreshVisuals();
						}
					} catch(NullPointerException npe){/*ignore*/}
				}
				break;		
			case RonmodelPackage.HL_PLACE__INCOMING:
				refreshTargetConnections();				
				//refreshVisuals of all pre-transitions (checkEnabled)
				for (Object o: getHLPlace().getIncoming()){
					try{
						HLTransition t=(HLTransition)((HLArc)o).getSource();
						for (Object o1: t.eAdapters()){
							if (o1 instanceof HLTransitionEditPart)
								((HLTransitionEditPart)o1).refreshVisuals();
						}
					} catch(NullPointerException npe){/*ignore*/}
				}
				break;
			case RonmodelPackage.HL_PLACE__TOKENS:
				for(int i=0; i<this.getAllPlaceEditParts().size();i++){
					this.getAllPlaceEditParts().get(i).refreshChildren();
				}
				for (Object o : this.getSourceConnections())
				{
					((ConnectionEditPart)o).getTarget().refresh();
				}
			case RonmodelPackage.HL_PLACE__NAME:
			case RonmodelPackage.HL_PLACE__X:
			case RonmodelPackage.HL_PLACE__Y:	
				this.refreshVisuals();	
				break;
			default: 
			super.notifyChanged(notification);
		}
	}


	/* (non-Javadoc)
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
	 */
	protected void createEditPolicies() {
		// install the edit policy to handle connection creation
		installEditPolicy( EditPolicy.GRAPHICAL_NODE_ROLE, 
				new HLPlaceEditPolicy() );
		// install the edit policy to handle deletion
		installEditPolicy(EditPolicy.COMPONENT_ROLE,
			new HLNodeComponentEditPolicy());
		
		installEditPolicy(EditPolicy.LAYOUT_ROLE, 
				new HLPlaceXYLayoutEditPolicy());
	}
	
	/**
	 * refreshing the mouse point according to the request
	 * 
	 * @param point
	 */
	protected Point calculatePoint(Request req) {
		Point p = new Point(0, 0);
		if (req instanceof CreateConnectionRequest) {
			CreateConnectionRequest r = (CreateConnectionRequest) req;
			p = r.getLocation();
		} else if (req instanceof ReconnectRequest) {
			ReconnectRequest r2 = (ReconnectRequest) req;
			p = r2.getLocation();
		}
		return p;
	}// calculatePoint
	
	/* (non-Javadoc)
	 * @see org.eclipse.gef.NodeEditPart#getSourceConnectionAnchor(org.eclipse.gef.ConnectionEditPart)
	 */
	//@Override
	public ConnectionAnchor getSourceConnectionAnchor(ConnectionEditPart connection) {
		ConnectionAnchor sourceAnchor;
		
		int anchorNr =  ((HLArc) connection.getModel()).getAnchorNrS();
		if (anchorNr!=-1)
			sourceAnchor = getHLPlaceFigure().getSourceConnectionAnchors().get(anchorNr);
		else
			sourceAnchor= getHLPlaceFigure().getSourceCBConnectionAnchor();
		return sourceAnchor;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.NodeEditPart#getSourceConnectionAnchor(org.eclipse.gef.Request)
	 */
	//@Override
	public ConnectionAnchor getSourceConnectionAnchor(Request request) {		
		//return getHLPlaceFigure().getSourceConnectionAnchor();
		Point mousePoint = calculatePoint(request);		
		return getHLPlaceFigure().getSourceConnectionAnchorAt(mousePoint); 
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.NodeEditPart#getTargetConnectionAnchor(org.eclipse.gef.ConnectionEditPart)
	 */
	//@Override
	public ConnectionAnchor getTargetConnectionAnchor(ConnectionEditPart connection) {		
		ConnectionAnchor targetAnchor;

		int anchorNr = ((HLArc) connection.getModel()).getAnchorNrT();			
		if (anchorNr!=-1)
			targetAnchor = getHLPlaceFigure().getTargetConnectionAnchors().get(anchorNr);
		else
			targetAnchor = getHLPlaceFigure().getTargetCBConnectionAnchor();
		return targetAnchor;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.NodeEditPart#getTargetConnectionAnchor(org.eclipse.gef.Request)
	 */
	//@Override
	public ConnectionAnchor getTargetConnectionAnchor(Request request) {		
		Point mousePoint = calculatePoint(request);		
		return getHLPlaceFigure().getTargetConnectionAnchorAt(mousePoint); 
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.editparts.AbstractEditPart#refreshVisuals()
	 */
	@Override
	protected void refreshVisuals() {
		Point loc = new Point( getHLPlace().getX(), getHLPlace().getY() );
		Dimension size = new Dimension(-1,-1);
		Rectangle r = new Rectangle(loc ,size);
		((GraphicalEditPart) getParent()).setLayoutConstraint(
			this,getFigure(),r);
	}
	
	

	/* (non-Javadoc)
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#activate()
	 */
	@SuppressWarnings("unchecked")
	public void activate()
	{
		if (isActive())
			return;

		// start listening for changes in the model
		((Notifier)getHLPlace()).eAdapters().add(this);

		super.activate();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#deactivate()
	 */
	public void deactivate()
	{
		if (!isActive())
			return;

		// stop listening for changes in the model
		((Notifier)getHLPlace()).eAdapters().remove(this);

		super.deactivate();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.emf.common.notify.Adapter#getTarget()
	 */
	public Notifier getTarget() {
		return target;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.emf.common.notify.Adapter#isAdapterForType(java.lang.Object)
	 */
	public boolean isAdapterForType(Object type) {
		return type.equals( getModel().getClass() );
	}


	/* (non-Javadoc)
	 * @see org.eclipse.emf.common.notify.Adapter#setTarget(org.eclipse.emf.common.notify.Notifier)
	 */
	public void setTarget(Notifier newTarget) {
		target = newTarget;
	}


	/* (non-Javadoc)
	 * @see org.eclipse.core.runtime.IAdaptable#getAdapter(java.lang.Class)
	 */
	public Object getAdapter(Class key) {
		if (IPropertySource.class == key) {
			return getPropertySource();
		}
		return super.getAdapter( key );
	}
	

	
	/* (non-Javadoc)
	 * @see com.ibm.itso.sal330r.gefdemo.edit.NetworkElementEditPart#getPropertySource()
	 */
	protected IPropertySource getPropertySource() {
		if( propertySource == null ) {
			propertySource = new EObjectPropertySource( getHLPlace() );
		}
		return propertySource;

	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
	 */
	protected IFigure createFigure() {
		HLPlaceFigure figure = new HLPlaceFigure();
		if(this.getHLPlace().getType().getValue() == HLPlaceType.NET){
			figure.setEclipseForegroundColor(ColorConstants.darkGreen);
		}else{
			figure.setEclipseForegroundColor(ColorConstants.darkBlue);
		}
		return figure;
	}
	
	
	
	@Override
	public boolean understandsRequest(Request req) {
		final Object type = req.getType();
		return type.equals(AddTokenAction.ID)
		|| super.understandsRequest(req);
	}
	
	
	
	@Override
	public Command getCommand(Request request) {
		if (request.getType().equals(AddTokenAction.ID)) {
			if (((HLPlace)getModel()).getType().equals(HLPlaceType.NET_LITERAL)) {
				final ObjectNet newToken = RONModelManager.getFactory()
						.createObjectNet();
				return new CreateHLTokenCommand((HLPlace)getModel(), newToken);
			} else 
			if (((HLPlace)getModel()).getType().equals(
					HLPlaceType.RULE_LITERAL)) {
				final Rule newToken = RONModelManager.getFactory().createRule();
				LHS lhs = RONModelManager.getFactory().createLHS();
				lhs.setName("LHS");
				newToken.setLhs(lhs);
				RHS rhs = RONModelManager.getFactory().createRHS();
				rhs.setName("RHS");
				newToken.setRhs(rhs);
				NAC nac = RONModelManager.getFactory().createNAC();
				nac.setName("NAC 1");
				newToken.getNacs().add(nac);
				return new CreateHLTokenCommand((HLPlace)getModel(), newToken, RuleIDManager.getNextID());
			}
		}
		return super.getCommand(request);
	}
	
	
	/**
	 * This handles the direct edit and open request sent by the SelectionTool.
	 */
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.EditPart#performRequest(org.eclipse.gef.Request)
	 */
	@Override
	public void performRequest(final Request req) {
		if (req.getType().equals(RequestConstants.REQ_OPEN)) {
			//getParent().performRequest(req);
			(((MultiViewerPageViewer)getViewer()).getHostPage()).getActionRegistry().getAction(AddTokenAction.ID).run();
		}
	}
	
}
