package roneditor.ronview.edit.policies;

import org.eclipse.draw2d.ConnectionAnchor;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.requests.CreateConnectionRequest;
import org.eclipse.gef.requests.ReconnectRequest;

import roneditor.ronview.anchors.HLAnchor;
import roneditor.ronview.commands.ConnectionHLPlaceHLTransCommand;
import roneditor.ronview.commands.ConnectionHLTransHLPlaceCommand;
import roneditor.ronview.commands.ReconnectHLArcCommand;
import roneditor.ronview.edit.parts.HLPlaceEditPart;
import ronmodel.HLArc;
import ronmodel.HLPlace;
import ronmodel.HLTransition;


/**
 * EditPolicy for a HLPlace.
 */
public class HLPlaceEditPolicy 
	extends HLNodeEditPolicy {

	/* (non-Javadoc)
	 * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy#getConnectionCompleteCommand(org.eclipse.gef.requests.CreateConnectionRequest)
	 */
	protected Command getConnectionCompleteCommand(CreateConnectionRequest request) {
		Command command = request.getStartCommand();
		ConnectionHLTransHLPlaceCommand commandTP;
		if (command instanceof ConnectionHLTransHLPlaceCommand) {	
			// HLArc ConnectionComplete
			commandTP = (ConnectionHLTransHLPlaceCommand)command;	
			if (commandTP.getSource() == getHLPlace()) return null;
			commandTP.setTarget(getHLPlace());
			if (thereIsAlreadyHLArc(commandTP)) return null;
			ConnectionAnchor anchor = ((HLPlaceEditPart) getHost())
			.getTargetConnectionAnchor(request);
			if (anchor instanceof HLAnchor)
				commandTP.setAnchorNrT(((HLAnchor)anchor).getId());
			else
				commandTP.setAnchorNrT(-1);
			//request.setStartCommand(command);
		} else return null; //not possible
		if (!isAllowedArc(commandTP.getSource(), commandTP.getTarget(),false))
			return null;		
		return command;
	}
	
	protected HLPlace getHLPlace() {
		return (HLPlace)getHost().getModel();
	}
	
	protected HLPlaceEditPart getHLPlaceEditPart() {
		return (HLPlaceEditPart)getHost();
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy#getConnectionCreateCommand(org.eclipse.gef.requests.CreateConnectionRequest)
	 */
	protected Command getConnectionCreateCommand(CreateConnectionRequest request) {		
		Object reqObj = request.getNewObject();
		Command command = null;
		if (reqObj instanceof HLArc) {
			ConnectionHLPlaceHLTransCommand cmd = new ConnectionHLPlaceHLTransCommand();
			cmd.setHLArc((HLArc)reqObj);
			cmd.setSource(getHLPlace());
			ConnectionAnchor anchor = ((HLPlaceEditPart) getHost())
			.getSourceConnectionAnchor(request);
			if (anchor instanceof HLAnchor)
				cmd.setAnchorNrS(((HLAnchor)anchor).getId());
			else
				cmd.setAnchorNrS(-1);
			command= cmd;
		}			
		request.setStartCommand(command);
		return command;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy#getReconnectTargetCommand(org.eclipse.gef.requests.ReconnectRequest)
	 */
		protected Command getReconnectTargetCommand(ReconnectRequest request) {
		HLArc onArc = (HLArc) request.getConnectionEditPart().getModel();
		
		if (onArc.getTarget() instanceof HLTransition) {			
			return null;
		}			
		ReconnectHLArcCommand command = new ReconnectHLArcCommand(onArc);
		ConnectionAnchor anchor = ((HLPlaceEditPart) getHost()).getTargetConnectionAnchor(request);
		if (anchor instanceof HLAnchor)
			command.setNewAnchorNrT(((HLAnchor)anchor).getId());
		else
			command.setNewAnchorNrT(-1);
		command.setNewTarget(getHLPlace());
		if (thereIsAlreadyHLArc(command)) return null;
		if (nothingChanged(command)) return null;
		if (!isAllowedArc(onArc.getSource(),(HLPlace)getHost().getModel(),true))
			return null;
		return command;
	}

		/* (non-Javadoc)
		 * @see org.eclipse.gef.editpolicies.GraphicalNodeEditPolicy#getReconnectSourceCommand(org.eclipse.gef.requests.ReconnectRequest)
		 */
		protected Command getReconnectSourceCommand(ReconnectRequest request) {
			HLArc onArc = (HLArc) request.getConnectionEditPart().getModel();
			
			if (onArc.getSource() instanceof HLTransition) {			
				return null;
			}				
			ReconnectHLArcCommand command = new ReconnectHLArcCommand(onArc);
			ConnectionAnchor anchor = ((HLPlaceEditPart) getHost())
					.getSourceConnectionAnchor(request);
			if (anchor instanceof HLAnchor)
				command.setNewAnchorNrS(((HLAnchor)anchor).getId());
			else
				command.setNewAnchorNrS(-1);
			command.setNewSource(getHLPlace());
			if (thereIsAlreadyHLArc(command)) return null;
			if (nothingChanged(command)) return null;
			if (!isAllowedArc((HLPlace)getHost().getModel(),onArc.getTarget(),true))
				return null;
			return command;
		}
		
		
}
