package roneditor.ronview.edit.policies;

import org.eclipse.draw2d.FreeformViewport;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editparts.ScalableFreeformRootEditPart;
import org.eclipse.gef.editpolicies.NonResizableEditPolicy;
import org.eclipse.gef.editpolicies.XYLayoutEditPolicy;
import org.eclipse.gef.requests.CreateRequest;
import org.eclipse.jface.viewers.StructuredSelection;

import roneditor.commands.MoveLabelCommand;
import roneditor.edit.parts.LabelEditPart;
import roneditor.ronview.commands.CreateHLPlaceCommand;
import roneditor.ronview.commands.CreateHLTransitionCommand;
import roneditor.ronview.commands.SetHLConstraintCommand;
import roneditor.ronview.figures.HLPlaceFigure;
import roneditor.ronview.figures.HLTransitionFigure;
import ronmodel.HLNode;
import ronmodel.HLPlace;
import ronmodel.HLTransition;
import ronmodel.RON;

/**
 * EditPolicy for XY-Layout on RONEditParts.
 * 
 * @author Tony Modica
 */
public class RONXYLayoutEditPolicy extends XYLayoutEditPolicy {

	/**
	 * @return A command that creates a new object in the parent model object of
	 *         this host.
	 */
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.editpolicies.LayoutEditPolicy#getCreateCommand(org.eclipse.gef.requests.CreateRequest)
	 */
	@Override
	protected Command getCreateCommand(final CreateRequest request) {
		final Object newObjectType = request.getNewObjectType();

		// Get Point that contains the upper left coordiante of the visible part
		// of the ScalableFreeformRootEditPart
		final ScalableFreeformRootEditPart rootPart = (ScalableFreeformRootEditPart) getHost()
				.getRoot();
		final FreeformViewport viewPort = (FreeformViewport) rootPart
				.getFigure();
		final Point bias = viewPort.getClientArea().getLocation();

		//HINT Command fuer neue Kindelemente des RONs hier erzeugen
		Command createCommand=null;
		if( newObjectType == HLTransition.class ) {
			CreateHLTransitionCommand create = new CreateHLTransitionCommand();
			create.setHLTransition((HLTransition)request.getNewObject());
			create.setRON((RON)getHost().getModel());
			create.setX(request.getLocation().x);
			create.setY(request.getLocation().y);
			create.setLabelX(0);
			create.setLabelY(HLTransitionFigure.TRANSITION_FIGURE_SIZE+3);
			create.setLabel("create a HLTransition");
			createCommand= create;
		} else if( newObjectType == HLPlace.class ) {
			CreateHLPlaceCommand create = new CreateHLPlaceCommand();
			create.setRON((RON)getHost().getModel());
			create.setHLPlace((HLPlace)request.getNewObject());			
			create.setX(request.getLocation().x);
			create.setY(request.getLocation().y);
			create.setLabelX(0);
			create.setLabelY(HLPlaceFigure.getESize()+3);
			create.setLabel("create a HLPlace");
			createCommand= create;
			
		}
		return createCommand;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.editpolicies.LayoutEditPolicy#getDeleteDependantCommand(org.eclipse.gef.Request)
	 */
	@Override
	protected Command getDeleteDependantCommand(@SuppressWarnings("unused")
	final Request request) {
		return null;
	}

	/**
	 * @return A NonResizableEditPolicy to prevent the new EditPart being
	 *         resized by the user.
	 */
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.editpolicies.LayoutEditPolicy#createChildEditPolicy(org.eclipse.gef.EditPart)
	 */
	@Override
	protected EditPolicy createChildEditPolicy(@SuppressWarnings("unused")
	final EditPart child) {
		return new NonResizableEditPolicy();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.gef.editpolicies.XYLayoutEditPolicy#getMinimumSizeFor(org.eclipse.gef.GraphicalEditPart)
	 */
	@Override
	protected Dimension getMinimumSizeFor(final GraphicalEditPart child) {
		return child.getContentPane().getMinimumSize();
	}

	/**
	 * @return A MoveCommand with the model of the host EditPart and the
	 *         constraint of its new delta set.
	 */
	@Override
	protected Command createChangeConstraintCommand(final EditPart child,
			final Object constraint) {
		if (child instanceof LabelEditPart) {
			MoveLabelCommand moveCommand = new MoveLabelCommand();
			HLNode onNode = (HLNode)((LabelEditPart)child).getCastedModel();
			moveCommand.setNode(onNode);
			// Labelposition nur verndern, wenn es als einzigestes Element ausgewhlt ist.
			// Eventuell sollte man dies schon frher in der Kette der Aufrufe abfangen?
			if (((StructuredSelection)(child.getViewer().getSelectionManager().getSelection())).toList().size() <= 1) {
				// Position relativ zur PlaceFigure			
				moveCommand.setNewPosition(((Rectangle)constraint).getLocation().translate(-onNode.getX(), -onNode.getY()));
			} else {
				// Position relativ zur PlaceFigure			
				moveCommand.setNewPosition(new Point(onNode.getLabelX(), onNode.getLabelY()));
			}
			return moveCommand;
		} else {
			SetHLConstraintCommand locationCommand = new SetHLConstraintCommand();
			locationCommand.setHLNode((HLNode)child.getModel());
			locationCommand.setNewPosition(((Rectangle)constraint).getLocation());
			return locationCommand;
		}
	}
	
}
