package roneditor.ronview.figures;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.draw2d.ChopboxAnchor;
import org.eclipse.draw2d.ConnectionAnchor;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.PositionConstants;
import org.eclipse.draw2d.RectangleFigure;
import org.eclipse.draw2d.XYLayout;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;

import roneditor.ronview.anchors.HLAnchor;
import ronmodel.HLTransitionKind;

/**
 * Represents the graphical layout of a place.
 */
public class HLTransitionFigure extends RectangleFigure
{
	protected ChopboxAnchor incomingCBConnectionAnchor;
	protected ChopboxAnchor outgoingCBConnectionAnchor;
	protected List<HLAnchor> incomingConnectionAnchors = new ArrayList<HLAnchor>();
	protected List<HLAnchor> outgoingConnectionAnchors = new ArrayList<HLAnchor>();
	static public final int TRANSITION_FIGURE_SIZE = 40;
	protected Label transitionKindLabel;
	
	public HLTransitionFigure() {
		this.setOutline(true);
		setLayoutManager(new XYLayout());
		this.setFill(true);
		
		transitionKindLabel = new Label("");
		final FontData data = new FontData();
		data.setStyle(SWT.BOLD);
		data.setHeight(7);
		Font f = new Font(null, data);
		transitionKindLabel.setFont(f);
		this.add(transitionKindLabel);

//		t = new TransitionFigure();
		incomingCBConnectionAnchor = new ChopboxAnchor(this);
		outgoingCBConnectionAnchor = new ChopboxAnchor(this);
		this.createAnchors();
	}
	

	
	public void setColor(Color color){
		this.setBackgroundColor(color);
	}
	
	public void validate(){		
		getLayoutManager().setConstraint(transitionKindLabel, new Rectangle(TRANSITION_FIGURE_SIZE-transitionKindLabel.getSize().width-1, 0,
				-1, -1));
		super.validate();
		
		Dimension d = getPreferredSize(-1, -1);	
		this.setMaximumSize(d);
		this.setMinimumSize(d);
		this.setPreferredSize(d);
		getLayoutManager().setConstraint(this, new Rectangle(0, 0,
				TRANSITION_FIGURE_SIZE, TRANSITION_FIGURE_SIZE));
	}
	
	
	public ChopboxAnchor getSourceCBConnectionAnchor(){
		return outgoingCBConnectionAnchor;
	}
	public ChopboxAnchor getTargetCBConnectionAnchor(){
		return incomingCBConnectionAnchor;
	}
	
	
	/* (non-Javadoc)
	 * @see org.eclipse.draw2d.Figure#useLocalCoordinates()
	 */
	protected boolean useLocalCoordinates() {
		return true;
	}

	
	/* (non-Javadoc)
	 * @see org.eclipse.draw2d.IFigure#getMinimumSize(int, int)
	 */
	public Dimension getMinimumSize(int wHint, int hHint) {		
		return getPreferredSize();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.draw2d.IFigure#getPreferredSize(int, int)
	 */
	@Override
	public Dimension getPreferredSize(int wHint, int hHint) {
	
			return new Dimension(TRANSITION_FIGURE_SIZE, TRANSITION_FIGURE_SIZE);
	}

		
	
	/**
	 * This method creates the PAnchorPoints at the Node
	 */
	protected void createAnchors() {

		// 10 anchors around the transition figure
		Dimension dF = new Dimension(TRANSITION_FIGURE_SIZE,TRANSITION_FIGURE_SIZE);
		Point[] connectionPoints = new Point[32];

		connectionPoints[0] = new Point(dF.width * 1 / 8, 0);
		connectionPoints[12] = new Point(dF.width * 2 / 8, 0);
		connectionPoints[1] = new Point(dF.width * 3 / 8, 0);
		connectionPoints[13] = new Point(dF.width * 4 / 8, 0);
		connectionPoints[2] = new Point(dF.width * 5 / 8, 0);
		connectionPoints[14] = new Point(dF.width * 6 / 8, 0);
		connectionPoints[3] = new Point(dF.width * 7 / 8, 0);

		connectionPoints[15] = new Point(0, dF.height * 1 / 8);
		connectionPoints[16] = new Point(0, dF.height * 2 / 8);
		connectionPoints[4] = new Point(0, dF.height * 3 / 8);
		connectionPoints[17] = new Point(0, dF.height * 4 / 8);
		connectionPoints[6] = new Point(0, dF.height * 5 / 8);
		connectionPoints[18] = new Point(0, dF.height * 6 / 8);
		connectionPoints[19] = new Point(0, dF.height * 7 / 8);

		connectionPoints[20] = new Point(dF.width, dF.height * 1 / 8);
		connectionPoints[21] = new Point(dF.width, dF.height * 2 / 8);
		connectionPoints[5] = new Point(dF.width, dF.height * 3 / 8);
		connectionPoints[22] = new Point(dF.width, dF.height * 4 / 8);
		connectionPoints[7] = new Point(dF.width, dF.height * 5 / 8);
		connectionPoints[23] = new Point(dF.width, dF.height * 6 / 8);
		connectionPoints[24] = new Point(dF.width, dF.height * 7 / 8);

		connectionPoints[8] = new Point(dF.width * 1 / 8, dF.height);
		connectionPoints[25] = new Point(dF.width * 2 / 8, dF.height);
		connectionPoints[9] = new Point(dF.width * 3 / 8, dF.height);
		connectionPoints[26] = new Point(dF.width * 4 / 8, dF.height);
		connectionPoints[10] = new Point(dF.width * 5 / 8, dF.height);
		connectionPoints[27] = new Point(dF.width * 6 / 8, dF.height);
		connectionPoints[11] = new Point(dF.width * 7 / 8, dF.height);
		
		
		// die 4 Ecken
		connectionPoints[28] = new Point(0, 0);
		connectionPoints[29] = new Point(0, dF.height);
		connectionPoints[30] = new Point(dF.width , 0);
		connectionPoints[31] = new Point(dF.width, dF.height);

		HLAnchor tempA;
		for (int i = 0; i < 32; i++) {
			tempA = new HLAnchor(this);
			tempA.setRelativeDist(connectionPoints[i]);
			tempA.setId(i);
			incomingConnectionAnchors.add(tempA);
			outgoingConnectionAnchors.add(tempA);

		}
	}

	/**
	 * This method refreshes the Position of the AnchorPoints at the Node as
	 * soon as there has been occured a changing of the dimension
	 */
	protected void refreshAnchors() {

		Dimension dF = getPreferredSize(-1, -1);
		Point[] connectionPoints = new Point[32];

		connectionPoints[0] = new Point(dF.width * 1 / 8, 0);
		connectionPoints[12] = new Point(dF.width * 2 / 8, 0);
		connectionPoints[1] = new Point(dF.width * 3 / 8, 0);
		connectionPoints[13] = new Point(dF.width * 4 / 8, 0);
		connectionPoints[2] = new Point(dF.width * 5 / 8, 0);
		connectionPoints[14] = new Point(dF.width * 6 / 8, 0);
		connectionPoints[3] = new Point(dF.width * 7 / 8, 0);

		connectionPoints[15] = new Point(0, dF.height * 1 / 8);
		connectionPoints[16] = new Point(0, dF.height * 2 / 8);
		connectionPoints[4] = new Point(0, dF.height * 3 / 8);
		connectionPoints[17] = new Point(0, dF.height * 4 / 8);
		connectionPoints[6] = new Point(0, dF.height * 5 / 8);
		connectionPoints[18] = new Point(0, dF.height * 6 / 8);
		connectionPoints[19] = new Point(0, dF.height * 7 / 8);

		connectionPoints[20] = new Point(dF.width, dF.height * 1 / 8);
		connectionPoints[21] = new Point(dF.width, dF.height * 2 / 8);
		connectionPoints[5] = new Point(dF.width, dF.height * 3 / 8);
		connectionPoints[22] = new Point(dF.width, dF.height * 4 / 8);
		connectionPoints[7] = new Point(dF.width, dF.height * 5 / 8);
		connectionPoints[23] = new Point(dF.width, dF.height * 6 / 8);
		connectionPoints[24] = new Point(dF.width, dF.height * 7 / 8);

		connectionPoints[8] = new Point(dF.width * 1 / 8, dF.height);
		connectionPoints[25] = new Point(dF.width * 2 / 8, dF.height);
		connectionPoints[9] = new Point(dF.width * 3 / 8, dF.height);
		connectionPoints[26] = new Point(dF.width * 4 / 8, dF.height);
		connectionPoints[10] = new Point(dF.width * 5 / 8, dF.height);
		connectionPoints[27] = new Point(dF.width * 6 / 8, dF.height);
		connectionPoints[11] = new Point(dF.width * 7 / 8, dF.height);
		
		
		// die 4 Ecken
		connectionPoints[28] = new Point(0, 0);
		connectionPoints[29] = new Point(0, dF.height);
		connectionPoints[30] = new Point(dF.width , 0);
		connectionPoints[31] = new Point(dF.width, dF.height);

		for (int i = 0; i < 32; i++) {
			incomingConnectionAnchors.get(i).setRelativeDist(
					connectionPoints[i]);
			outgoingConnectionAnchors.get(i).setRelativeDist(
					connectionPoints[i]);
		}
	  }
	

	/**
	 * Returns a list of anchors for source connections
	 * 
	 * @return
	 */
	public List<HLAnchor> getSourceConnectionAnchors() {
		return outgoingConnectionAnchors;
	}
	
	/**
	 * Returns a list of anchors for target connections.
	 * 
	 * @return
	 */
	public List<HLAnchor> getTargetConnectionAnchors() {
		return incomingConnectionAnchors;
	}
	
	/**
	 * Detection of the closest source connection anchor around the mouse
	 * pointer
	 * 
	 * @return the anchor which is closest to the actual mouse position
	 */
	public ConnectionAnchor getSourceConnectionAnchorAt(Point mousePoint) {
		return getClosestConnectionAnchorAt(mousePoint,
				getSourceConnectionAnchors(),getSourceCBConnectionAnchor());
	}

	/**
	 * Detection of the closest target connection anchor around the mouse
	 * pointer
	 * 
	 * @return the anchor which is closest to the actual mouse position
	 */
	public ConnectionAnchor getTargetConnectionAnchorAt(Point mousePoint) {
		return getClosestConnectionAnchorAt(mousePoint,
				getTargetConnectionAnchors(),getTargetCBConnectionAnchor());
	}

	/**
	 * Calculating the minimal distant anchor regarding to the mouse pointer
	 * 
	 * @param p
	 * @param connectionAnchors
	 * @return the closest anchor around the point p
	 */
	static public ConnectionAnchor getClosestConnectionAnchorAt(Point p,
			List<HLAnchor> connectionAnchors, ChopboxAnchor cba) {

		// initializing variables
		ConnectionAnchor closest = null;
		long min = Long.MAX_VALUE;
		int anz_Anchors = connectionAnchors.size();
		Point p2 = new Point(0, 0);

		// checking each anchor point and taking the best one
		for (int i = 0; i < anz_Anchors; i++) {
			HLAnchor a = connectionAnchors.get(i);

			// absolut coordinates of the anchor
			p2 = a.getLocation(null);

			long d = p.getDistance2(p2);
			if (d < min) {
				min = d;
				closest = a;
			}
		}
		
		//chopbox anchor
		p2 = cba.getReferencePoint();	
		long d = p.getDistance2(p2);
		if (d < min+50) {
			min = d;
			closest = cba;
		}
		return closest;
	}

	public void setTransitionKind(HLTransitionKind kind) {
		switch (kind.getValue()) {
		case HLTransitionKind.FIRE : 
			transitionKindLabel.setText(kind.getName().substring(0,4));
			break;
		case HLTransitionKind.STANDARD : 
			transitionKindLabel.setText("");
			break;
		case HLTransitionKind.APPLYRULE : 
			transitionKindLabel.setText(kind.getName().substring(0,5));
			break;
		case HLTransitionKind.SPLIT : 
			transitionKindLabel.setText(kind.getName().substring(0,5));
			break;
		}
		
		validate();
	}



}