/**
 * 
 */
package roneditor.ruleview.actions;

import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

import org.eclipse.gef.commands.Command;
import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.ui.IEditorPart;

import roneditor.ruleview.RulePage;
import roneditor.ruleview.commands.CreateNacCommand;
import ronmodel.NAC;
import ronmodel.RonmodelFactory;
import ronmodel.Rule;
import ronmodel.impl.RonmodelFactoryImpl;

/**
 * @author Mario
 *
 */
public class CreateNacAction extends SelectionAction{

	static public final String ID = "Create Nac Action";
	private RulePage rulePage;
	private Rule rule;
	
	public CreateNacAction(RulePage _rulePage, IEditorPart part) {
		super(part);
		setDescription("Create Nac");
		setText("Create Nac");
		setId(ID);
		this.rulePage = _rulePage;
		this.rule = _rulePage.getRule();
	}
	
	public void run(){
		execute(getCommand());
	}
	
	public Command getCommand(){
		
		
		CreateNacCommand cnc = new CreateNacCommand();
		RonmodelFactory ronFactory = new RonmodelFactoryImpl();
		NAC newNac = ronFactory.createNAC();
		newNac.setName("NAC "+getFirstFreeNacNumber(rule).toString());
		cnc.setNewNac(newNac);
		cnc.setRule(rule);
		cnc.setRulePage(rulePage);
		
		
		return cnc;
	}

	
	private Integer getFirstFreeNacNumber(Rule rule) {
		List<NAC> nacs = rule.getNacs();
		FreeList fl = new FreeList();
		for (NAC nac:nacs) {
			StringTokenizer st= new StringTokenizer(nac.getName());
			// "NAC" berspringen
			if (st.hasMoreTokens()) {
				st.nextToken();
			}
			// nun sollte der nchste Token die Zahl beinhalten
			if (st.hasMoreTokens()) {
				try {
					fl.addNumber(Integer.valueOf(st.nextToken()));
				} catch (NumberFormatException e) {
					// dies sollte eigenlich nie der Fall sein, aber zur Sicherheit
					return 1;
				}
			} else {
				// dies sollte eigenlich nie der Fall sein, aber zur Sicherheit
				return 1;
			}
		}		
		return fl.getFirstFreeNumber();
	}
	
	/**
	 * Hilfsklasse um die kleineste  Nummer herauszufinden die noch frei ist
	 * 
	 * @author Mario
	 *
	 */
	private class FreeList {
		List<Boolean> freeList;

		/**
		 * Konstruktor
		 */
		public FreeList() {
			super();
			this.freeList = new ArrayList<Boolean>();
		}
		
		/**
		 * 
		 * @param i Nummer die schon vergeben ist (also nicht frei)
		 */
		public void addNumber(Integer i) {
			if (this.freeList.size() < i) {
				this.freeList.add(true);
				// Rekursion
				addNumber(i);
			} else {
				this.freeList.set(i-1, false);
			}
		}
		
		
		/**
		 * Aufrufen nachdem alle vergebenen Nummern mit addNumber eingetragen wurden.
		 * @return erste freie Nummer
		 */
		public Integer getFirstFreeNumber() {
			Integer i = 1;
			for (Boolean free: this.freeList) {
				if (free) {
					return i; 
				}
				i++;
			}
			return this.freeList.size()+1;
		}
		
		
	}
	
	protected boolean calculateEnabled() {
		return true;
	}	

	/* (non-Javadoc)
	 * @see org.eclipse.gef.ui.actions.WorkbenchPartAction#init()
	 */
	protected void init() {
		super.init();
		setId(ID);
	}
}

