package roneditor.ruleview.actions;

import java.util.List;

import org.eclipse.jface.action.Action;

import roneditor.ruleview.RulePage;
import ronmodel.NAC;

/**
 * This action is used by a {@link RulePage} and switches the NAC that is shown
 * in the first viewer.
 * 
 * @author Tony Modica
 */
public class SwitchNACAction extends Action {

	static public final String ID = "Switch NAC Action";

	private RulePage rulePage;

	public SwitchNACAction(final RulePage rulePage) {
		this.rulePage = rulePage;
		setId(ID);
		setText("Switch NAC");
		setDescription("Switch NAC");
		setToolTipText("Switch NAC");
	}

	/**
	 * Change the contents of the viewer for the NACs (the left-most) to the
	 * next element in the list of the rule's NACs. If the last NAC is being
	 * shown switch to the first in the list.
	 */
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.action.Action#run()
	 */
	@Override
	public void run() {
		final List nacs = rulePage.getRule().getNacs();
		final int actualShowing = nacs.lastIndexOf(rulePage
				.getViewersContents(0));
		final NAC newNACtoShow = (NAC) nacs.get((actualShowing + 1)
				% nacs.size());
		rulePage.setViewersContents(0, newNACtoShow);
		// angezeigten MappingNummern neu berechnen und setzen
		// roneditor.objectnetview.MappingCreationTool.calculateMapping(rulePage);
	}

	/**
	 * This action is only enabled for pages showing rules with at least 2 NACs.
	 */
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.action.Action#isEnabled()
	 */
	@Override
	public boolean isEnabled() {
		return rulePage.getRule().getNacs().size() > 1;
	}
}
