package roneditor.ruleview.commands;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.commands.CompoundCommand;

import roneditor.model.ModelCreationFactory;
import roneditor.objectnetview.commands.MappingONPlaceCommand;
import roneditor.objectnetview.commands.MappingONTransitionCommand;
import ronmodel.LHS;
import ronmodel.Mapping;
import ronmodel.ONNode;
import ronmodel.ONPlace;
import ronmodel.ONTransition;
import ronmodel.RHS;
import ronmodel.Rule;

/**
 * @author conny
 * 
 * Command to copy the LHS to the RHS. 
 * It is used by CopyLhsToRhsAction.
 */

public class CopyLhsToRhsCommand extends Command {
	private Rule rule;
	private RHS currentRhs;
	private List lhsArcs, lhsNodes;
	private CompoundCommand cCommand;

	public CopyLhsToRhsCommand() {
		setLabel("Copy LHS To RHS");
	}

	public void execute() {		
//		copy of lhs
		LHS copy = (LHS) EcoreUtil.copy(this.rule.getLhs());
		
		
		// get the current rhs
		this.currentRhs = (RHS) this.rule.getRhs();
		
		// get the nodes and arcs of the lhs copy
		this.lhsNodes = new ArrayList<Object>(copy.getNodes());
		this.lhsArcs = new ArrayList<Object>(copy.getArcs());
		this.cCommand = new CompoundCommand();
		
		
		
		
		// mappings nur erstellen, wenn auf der Zielseite noch keine sind, sonst kann es zu Konflikten kommen
		if (this.currentRhs.getMappings().isEmpty()){ 	
			// add nodes and arcs of the lhs copy to the rhs
			EList orgLhsNodes = this.rule.getLhs().getNodes();
			ModelCreationFactory mappingFactory = new ModelCreationFactory(Mapping.class);
			// Mapping hinzufgen		
			for (int i = 0; i < orgLhsNodes.size(); i++) {
				ONNode copyLhsNode = (ONNode)lhsNodes.get(i);
				this.currentRhs.getNodes().add(lhsNodes.get(i));
				
				Mapping newMapping= (Mapping)mappingFactory.getNewObject();
				if (copyLhsNode instanceof ONPlace) {
					MappingONPlaceCommand command = new MappingONPlaceCommand();
					command.setMapping(newMapping);
					command.setSource((ONPlace)orgLhsNodes.get(i));
					command.setTarget((ONPlace)copyLhsNode);
					cCommand.add(command);					
				} else {
					MappingONTransitionCommand command = new MappingONTransitionCommand();
					command.setMapping(newMapping);
					command.setSource((ONTransition)orgLhsNodes.get(i));
					command.setTarget((ONTransition)copyLhsNode);	
					cCommand.add(command);				
				}
			}
			cCommand.execute();
		} else {
			// wenn keine Mappings eingefgt werden, einfach alle Knoten einfgen
			this.currentRhs.getNodes().addAll(this.lhsNodes);			
		}
		this.currentRhs.getArcs().addAll(this.lhsArcs);
		copy = null;
		
		
	}


	public void undo() {
		//remove the arcs and nodes of LHS from the current RHS
		this.currentRhs.getArcs().removeAll(this.lhsArcs);
		this.currentRhs.getNodes().removeAll(this.lhsNodes);
		cCommand.undo();

	}

	/*
	 *  add nodes and arcs of the lhs copy to the current rhs
	 */
	public void redo() {
		this.currentRhs.getNodes().addAll(this.lhsNodes);
		this.currentRhs.getArcs().addAll(this.lhsArcs);
		cCommand.redo();
	}

	public void setRule(Rule _rule) {
		this.rule = _rule;
	}
}
