/**
 * <copyright>
 * </copyright>
 *
 * $Id: Rule.java,v 1.5 2007/07/21 14:02:56 calistar Exp $
 */
package ronmodel;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.emf.common.util.EList;


/**
 * <!-- begin-user-doc -->
 * A representation of the model object '<em><b>Rule</b></em>'.
 * <!-- end-user-doc -->
 *
 * <p>
 * The following features are supported:
 * <ul>
 *   <li>{@link ronmodel.Rule#getLhs <em>Lhs</em>}</li>
 *   <li>{@link ronmodel.Rule#getRhs <em>Rhs</em>}</li>
 *   <li>{@link ronmodel.Rule#getNacs <em>Nacs</em>}</li>
 *   <li>{@link ronmodel.Rule#getID <em>ID</em>}</li>
 * </ul>
 * </p>
 *
 * @see ronmodel.RonmodelPackage#getRule()
 * @model
 * @generated
 */
public interface Rule extends Token {
	
	
	MappingNumberList getMappingNumberList();
	
	
	/**
	 * Hilfsklasse zur Speicherung der Mappingnummern
	 * @author Mario
	 *
	 */
	public class MappingNumberList {
		List<Boolean> free;
		List<List<Mapping>> mappings;
		
		public MappingNumberList(Rule rule) {
			free = new ArrayList<Boolean>();
			mappings = new ArrayList<List<Mapping>>();
			List<Mapping> mappings = rule.getRhs().getMappings();
			for (Mapping m:mappings) {
				if (m.getMappingNr() != -1) {
					this.setMappingNr(m.getMappingNr(),m);
				}
			}
			List<NAC> nacs= rule.getNacs();
			for (NAC nac:nacs) {
				mappings = nac.getMappings();
				for (Mapping m:mappings) {
					if (m.getMappingNr() != -1) {
						this.setMappingNr(m.getMappingNr(),m);
					}
				}
			}
		}
		
		/**
		 * Trys to find an already given Mappingnumber for one of the Mappings of the List. 
		 * If Mappings dont have a Mappingnumber yet, then calculate the first free one.
		 * @param m List of Mappings that belong to ONNode
		 * @return Mappingnumber that should be used
		 */
		public Integer getMappingNumber(List<Mapping> m) {
			Integer index = indexOfMapping(m);
			
			if ((index != null) && !free.get(index)) {
				return index;
			} else {
				index= getFirstFreeIndex();
				if (index < free.size()) { 
					setNewMappings(index, m);
					free.set(index, false);
				} else {
					addNewMappings(m);
					free.add(false);					
				}
			}
			return index;
		}
		
		/**
		 * Insert the List of Mapping at index.
		 * @param index where the Mappings should be inserted
		 * @param m List of Mappings that belong to ONNode
		 */
		private void setNewMappings(Integer index, List<Mapping> m) {
			List<Mapping> newList = new ArrayList<Mapping>(m);
			mappings.set(index, newList);
		}
		
		/**
		 * Add the List of Mappings at the end.
		 * @param m List of Mappings that belong to ONNode
		 */
		private void addNewMappings(List<Mapping> m) {
			List<Mapping> newList = new ArrayList<Mapping>(m);
			mappings.add(newList);
		}
		
		/**
		 * 
		 * @param m List of Mappings the belong to ONNode
		 * @return Index where at least one Mapping of m is or null if no Mapping of m anywhere.
		 */
		private Integer indexOfMapping(List<Mapping> m) {
			for(int i= 0; i < mappings.size(); i++) {
				if (!free.get(i)) {
					for (Mapping mapping:mappings.get(i)) {
						for (Mapping mapping2:m) {
							if (mapping == mapping2) {
								addMappings(i,m);
								return i;
							}
							
						}
					}
				}
			}
			return null;
		}
		
		
		/**
		 * @param m List of Mappings the belong to ONNode
		 * @return Index where the Mapping is or null if not in List.
		 */
		public Integer indexOfMapping(Mapping m) {
			for(int i= 0; i < mappings.size(); i++) {
				if (!free.get(i)) {
					for (Mapping mapping:mappings.get(i)) {
						if (mapping == m) {
							return i;
						}					
					}
				}
			}
			return null;
		}
		
		/**
		 * Adds Mappings of m at index if they not allready there.
		 * @param index where to put the Mappings
		 * @param m List of Mapping that belong to ONNode.
		 */
		private void addMappings(Integer index, List<Mapping> m) {
			for (Mapping mapping:m) {
				if (!mappings.get(index).contains(mapping)) {
					mappings.get(index).add(mapping);
				}
			}
		}
		
		/**
		 * Index of the first "true" or actual size of the list, if all entrys false. 
		 * @return First free Index.
		 */
		private Integer getFirstFreeIndex() {
			for (int i= 0; i < free.size(); i++) {
				if (free.get(i)) return i; 
			}
			return free.size();
		}
		
		/**
		 * Removes Mapping m if its anywhere and when List empty after it has been removed, intern free status changed to true.
		 * @param m Mapping that should be removed
		 */
		public void removeMapping(Mapping m) {
			Integer index = indexOfMapping(m);
			if ((index != null) && !free.get(index)) {
				mappings.get(index).remove(m);
				if (mappings.get(index).isEmpty()) {
					free.set(index, true);
				}
			}
		}

		/**
		 * Insert the  Mapping at index.
		 * @param index where the Mappings should be inserted
		 * @param m Mapping that belong to ONNode
		 */
		public void setMappingNr(Integer index, Mapping m) {	
			// Listen vergrern wenn ntig
			while (free.size() <= index-1) {
				free.add(true);
				mappings.add(new ArrayList<Mapping>());
			}
			if (free.get(index-1)) {
				List<Mapping> newList = new ArrayList<Mapping>();
				newList.add(m);
				mappings.set(index-1, newList);
				free.set(index-1, false);
			} else {
				mappings.get(index-1).add(m);
			}
			
		}
		
		public void reset() {
			free = new ArrayList<Boolean>();
			mappings = new ArrayList<List<Mapping>>();
		}
		
		
	}
	
	/**
	 * Returns the value of the '<em><b>Lhs</b></em>' containment reference.
	 * It is bidirectional and its opposite is '{@link ronmodel.LHS#getRule <em>Rule</em>}'.
	 * <!-- begin-user-doc -->
	 * <p>
	 * If the meaning of the '<em>Lhs</em>' containment reference isn't clear,
	 * there really should be more of a description here...
	 * </p>
	 * <!-- end-user-doc -->
	 * @return the value of the '<em>Lhs</em>' containment reference.
	 * @see #setLhs(LHS)
	 * @see ronmodel.RonmodelPackage#getRule_Lhs()
	 * @see ronmodel.LHS#getRule
	 * @model opposite="rule" containment="true"
	 * @generated
	 */
	LHS getLhs();

	/**
	 * Sets the value of the '{@link ronmodel.Rule#getLhs <em>Lhs</em>}' containment reference.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @param value the new value of the '<em>Lhs</em>' containment reference.
	 * @see #getLhs()
	 * @generated
	 */
	void setLhs(LHS value);

	/**
	 * Returns the value of the '<em><b>Rhs</b></em>' containment reference.
	 * It is bidirectional and its opposite is '{@link ronmodel.RHS#getRule <em>Rule</em>}'.
	 * <!-- begin-user-doc -->
	 * <p>
	 * If the meaning of the '<em>Rhs</em>' containment reference isn't clear,
	 * there really should be more of a description here...
	 * </p>
	 * <!-- end-user-doc -->
	 * @return the value of the '<em>Rhs</em>' containment reference.
	 * @see #setRhs(RHS)
	 * @see ronmodel.RonmodelPackage#getRule_Rhs()
	 * @see ronmodel.RHS#getRule
	 * @model opposite="rule" containment="true"
	 * @generated
	 */
	RHS getRhs();

	/**
	 * Sets the value of the '{@link ronmodel.Rule#getRhs <em>Rhs</em>}' containment reference.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @param value the new value of the '<em>Rhs</em>' containment reference.
	 * @see #getRhs()
	 * @generated
	 */
	void setRhs(RHS value);

	/**
	 * Returns the value of the '<em><b>Nacs</b></em>' containment reference list.
	 * The list contents are of type {@link ronmodel.NAC}.
	 * It is bidirectional and its opposite is '{@link ronmodel.NAC#getRule <em>Rule</em>}'.
	 * <!-- begin-user-doc -->
	 * <p>
	 * If the meaning of the '<em>Nacs</em>' containment reference list isn't clear,
	 * there really should be more of a description here...
	 * </p>
	 * <!-- end-user-doc -->
	 * @return the value of the '<em>Nacs</em>' containment reference list.
	 * @see ronmodel.RonmodelPackage#getRule_Nacs()
	 * @see ronmodel.NAC#getRule
	 * @model type="ronmodel.NAC" opposite="rule" containment="true"
	 * @generated
	 */
	EList getNacs();

	/**
	 * Returns the value of the '<em><b>ID</b></em>' attribute.
	 * The default value is <code>"-1"</code>.
	 * <!-- begin-user-doc -->
	 * <p>
	 * If the meaning of the '<em>ID</em>' attribute isn't clear,
	 * there really should be more of a description here...
	 * </p>
	 * <!-- end-user-doc -->
	 * @return the value of the '<em>ID</em>' attribute.
	 * @see #setID(int)
	 * @see ronmodel.RonmodelPackage#getRule_ID()
	 * @model default="-1" id="true"
	 * @generated
	 */
	int getID();

	/**
	 * Sets the value of the '{@link ronmodel.Rule#getID <em>ID</em>}' attribute.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @param value the new value of the '<em>ID</em>' attribute.
	 * @see #getID()
	 * @generated
	 */
	void setID(int value);

} // Rule